@@ -94,7 +94,8 @@ def InterpretMachineType(machine_type,
                          custom_cpu,
                          custom_memory,
                          ext=True,
-                         vm_type=False):
+                         vm_type=False,
+                         confidential_vm=False):
   """Interprets the machine type for the instance.
 
   Args:
@@ -103,6 +104,8 @@ def InterpretMachineType(machine_type,
     custom_memory: amount of RAM memory in bytes for custom machine type,
     ext: extended custom machine type should be used if true,
     vm_type:  VM instance generation
+    confidential_vm: If True, default machine type is different for confidential
+      VMs.
 
   Returns:
     A string representing the URL naming a machine-type.
@@ -114,9 +117,12 @@ def InterpretMachineType(machine_type,
       custom machine type flags are used to generate a new instance.
   """
   # Setting the machine type
-  machine_type_name = constants.DEFAULT_MACHINE_TYPE
   if machine_type:
     machine_type_name = machine_type
+  elif confidential_vm:
+    machine_type_name = constants.DEFAULT_MACHINE_TYPE_FOR_CONFIDENTIAL_VMS
+  else:
+    machine_type_name = constants.DEFAULT_MACHINE_TYPE
 
   # Setting the specs for the custom machine.
   if custom_cpu or custom_memory or ext:
@@ -238,7 +244,9 @@ def CreateSchedulingMessage(messages,
                             restart_on_failure,
                             node_affinities=None,
                             min_node_cpu=None,
-                            location_hint=None):
+                            location_hint=None,
+                            maintenance_freeze_duration=None,
+                            maintenance_interval=None):
   """Create scheduling message for VM."""
   # Note: We always specify automaticRestart=False for preemptible VMs. This
   # makes sense, since no-restart-on-failure is defined as "store-true", and
@@ -265,6 +273,14 @@ def CreateSchedulingMessage(messages,
 
   if location_hint:
     scheduling.locationHint = location_hint
+
+  if maintenance_freeze_duration:
+    scheduling.maintenanceFreezeDurationHours = \
+      maintenance_freeze_duration // 3600  # sec to hour
+
+  if maintenance_interval:
+    scheduling.maintenanceInterval = messages.\
+      Scheduling.MaintenanceIntervalValueValuesEnum(maintenance_interval)
   return scheduling
 
 
@@ -299,6 +315,15 @@ def CreateConfidentialInstanceMessage(messages, enable_confidential_compute):
   return confidential_instance_config
 
 
+def CreateAdvancedMachineFeaturesMessage(messages,
+                                         enable_nested_virtualization=None,
+                                         threads_per_core=None):
+  """Create AdvancedMachineFeatures message for an Instance."""
+  return messages.AdvancedMachineFeatures(
+      enableNestedVirtualization=enable_nested_virtualization,
+      threadsPerCore=threads_per_core)
+
+
 def ParseDiskResource(resources, name, project, zone, type_):
   """Parses disk resources.
 
@@ -440,7 +465,7 @@ def GetScheduling(args,
                   client,
                   skip_defaults,
                   support_node_affinity=False,
-                  support_min_node_cpu=False,
+                  support_min_node_cpu=True,
                   support_location_hint=False):
   """Generate a Scheduling Message or None based on specified args."""
   node_affinities = None
@@ -457,6 +482,14 @@ def GetScheduling(args,
       args, 'maintenance_policy', 'preemptible', 'restart_on_failure') and
       not node_affinities):
     return None
+  freeze_duration = None
+  if hasattr(args, 'maintenance_freeze_duration') and args.IsSpecified(
+      'maintenance_freeze_duration'):
+    freeze_duration = args.maintenance_freeze_duration
+  maintenance_interval = None
+  if hasattr(args, 'maintenance_interval') and args.IsSpecified(
+      'maintenance_interval'):
+    maintenance_interval = args.maintenance_interval
   return CreateSchedulingMessage(
       messages=client.messages,
       maintenance_policy=args.maintenance_policy,
@@ -464,7 +497,9 @@ def GetScheduling(args,
       restart_on_failure=args.restart_on_failure,
       node_affinities=node_affinities,
       min_node_cpu=min_node_cpu,
-      location_hint=location_hint)
+      location_hint=location_hint,
+      maintenance_freeze_duration=freeze_duration,
+      maintenance_interval=maintenance_interval)
 
 
 def GetServiceAccounts(args, client, skip_defaults):
@@ -533,7 +568,7 @@ def CheckSpecifiedMachineTypeArgs(args, skip_defaults):
 
 
 def CreateMachineTypeUri(args, compute_client, resource_parser, project,
-                         location, scope):
+                         location, scope, confidential_vm=False):
   """Create a machine type URI for given args and instance reference."""
 
   machine_type = args.machine_type
@@ -548,7 +583,8 @@ def CreateMachineTypeUri(args, compute_client, resource_parser, project,
       custom_cpu=custom_cpu,
       custom_memory=custom_memory,
       ext=ext,
-      vm_type=vm_type)
+      vm_type=vm_type,
+      confidential_vm=confidential_vm)
 
   # Check to see if the custom machine type ratio is supported
   CheckCustomCpuRamRatio(compute_client, project, location, machine_type_name)
@@ -584,10 +620,14 @@ def GetTags(args, client):
   return None
 
 
-def GetLabels(args, client):
+def GetLabels(args, client, instance_properties=False):
+  """Gets labels for the instance message."""
+  labels_value = client.messages.Instance.LabelsValue
+  if instance_properties:
+    labels_value = client.messages.InstanceProperties.LabelsValue
   if args.labels:
-    return client.messages.Instance.LabelsValue(additionalProperties=[
-        client.messages.Instance.LabelsValue.AdditionalProperty(
+    return labels_value(additionalProperties=[
+        labels_value.AdditionalProperty(
             key=key, value=value)
         for key, value in sorted(six.iteritems(args.labels))
     ])
