@@ -70,6 +70,8 @@ _SECRET_COLLECTION = 'anthosevents.api.v1.namespaces.secrets'
 
 _CLOUD_RUN_RELATIVE_NAME = 'namespaces/cloud-run-system/cloudruns/cloud-run'
 
+_TRIGGERS_CRD_NAME = 'triggers.eventing.knative.dev'
+
 
 def Connect(conn_info):
   """Provide a AnthosEventsOperations instance to use.
@@ -232,12 +234,16 @@ class AnthosEventsOperations(object):
       v1alpha1_client: The v1alpha1 API client
       v1beta1_client: The v1beta1 API client
     """
+    self._client_from_crd_cache = {}
 
     self._api_version = api_version
 
-    # used for triggers and sources
+    # used for triggers
     self._client = v1beta1_client
 
+    # used for cloudsources
+    self._cloudsources_client = v1_client
+
     # used for operator
     self._operator_client = v1alpha1_client
 
@@ -264,16 +270,40 @@ class AnthosEventsOperations(object):
   def messages(self):
     return self._client.MESSAGES_MODULE
 
+  def ClientFromCrd(self, crd_name):
+    """Returns client based on customresource."""
+    if crd_name in self._client_from_crd_cache:
+      return self._client_from_crd_cache[crd_name]
+    crd = self.GetCustomResourceDefinition(crd_name)
+    if 'v1' in crd.getActiveVersions():
+      client = self._core_client
+    else:
+      # Whether 'v1beta1' crd or None when no crd found.
+      client = self._client
+    self._client_from_crd_cache[crd_name] = client
+    return client
+
+  def ListNamespaces(self):
+    """Returns a list of namespaces' names."""
+    request = (
+        self._core_client.MESSAGES_MODULE
+        .AnthoseventsApiV1NamespacesListRequest())
+    response = self._core_client.api_v1_namespaces.List(request)
+    return [item.metadata.name for item in response.items]
+
   def GetTrigger(self, trigger_ref):
     """Returns the referenced trigger."""
-    request = self.messages.AnthoseventsNamespacesTriggersGetRequest(
+    client = self.ClientFromCrd(_TRIGGERS_CRD_NAME)
+    messages = client.MESSAGES_MODULE
+
+    request = messages.AnthoseventsNamespacesTriggersGetRequest(
         name=trigger_ref.RelativeName())
     try:
       with metrics.RecordDuration(metric_names.GET_TRIGGER):
-        response = self._client.namespaces_triggers.Get(request)
+        response = client.namespaces_triggers.Get(request)
     except api_exceptions.HttpNotFoundError:
       return None
-    return trigger.Trigger(response, self.messages)
+    return trigger.Trigger(response, messages)
 
   def CreateTrigger(self, trigger_ref, source_obj, event_type, trigger_filters,
                     target_service, broker_name):
@@ -291,7 +321,9 @@ class AnthosEventsOperations(object):
     Returns:
       trigger.Trigger of the created trigger.
     """
-    trigger_obj = trigger.Trigger.New(self._client, trigger_ref.Parent().Name())
+    client = self.ClientFromCrd(_TRIGGERS_CRD_NAME)
+    messages = client.MESSAGES_MODULE
+    trigger_obj = trigger.Trigger.New(client, trigger_ref.Parent().Name())
     trigger_obj.name = trigger_ref.Name()
     if source_obj is not None:
       trigger_obj.dependency = source_obj
@@ -305,17 +337,17 @@ class AnthosEventsOperations(object):
 
     trigger_obj.subscriber = target_service
     trigger_obj.broker = broker_name
-    request = self.messages.AnthoseventsNamespacesTriggersCreateRequest(
+    request = messages.AnthoseventsNamespacesTriggersCreateRequest(
         trigger=trigger_obj.Message(),
         parent=trigger_ref.Parent().RelativeName())
     try:
       with metrics.RecordDuration(metric_names.CREATE_TRIGGER):
-        response = self._client.namespaces_triggers.Create(request)
+        response = client.namespaces_triggers.Create(request)
     except api_exceptions.HttpConflictError:
       raise exceptions.TriggerCreationError(
           'Trigger [{}] already exists.'.format(trigger_obj.name))
 
-    return trigger.Trigger(response, self.messages)
+    return trigger.Trigger(response, messages)
 
   def PollTrigger(self, trigger_ref, tracker):
     """Wait for trigger to be Ready == True."""
@@ -326,23 +358,37 @@ class AnthosEventsOperations(object):
 
   def ListTriggers(self, namespace_ref):
     """Returns a list of existing triggers in the given namespace."""
-    request = self.messages.AnthoseventsNamespacesTriggersListRequest(
+    client = self.ClientFromCrd(_TRIGGERS_CRD_NAME)
+    messages = client.MESSAGES_MODULE
+
+    request = messages.AnthoseventsNamespacesTriggersListRequest(
         parent=namespace_ref.RelativeName())
     with metrics.RecordDuration(metric_names.LIST_TRIGGERS):
-      response = self._client.namespaces_triggers.List(request)
-    return [trigger.Trigger(item, self.messages) for item in response.items]
+      response = client.namespaces_triggers.List(request)
+    return [trigger.Trigger(item, messages) for item in response.items]
 
   def DeleteTrigger(self, trigger_ref):
     """Deletes the referenced trigger."""
-    request = self.messages.AnthoseventsNamespacesTriggersDeleteRequest(
+    client = self.ClientFromCrd(_TRIGGERS_CRD_NAME)
+    messages = client.MESSAGES_MODULE
+
+    request = messages.AnthoseventsNamespacesTriggersDeleteRequest(
         name=trigger_ref.RelativeName())
     try:
       with metrics.RecordDuration(metric_names.DELETE_TRIGGER):
-        self._client.namespaces_triggers.Delete(request)
+        client.namespaces_triggers.Delete(request)
     except api_exceptions.HttpNotFoundError:
       raise exceptions.TriggerNotFound(
           'Trigger [{}] not found.'.format(trigger_ref.Name()))
 
+  def ClientFromSourceCrd(self, source_crd):
+    """Returns the correct client for the source."""
+    if 'v1' in source_crd.getActiveSourceVersions():
+      client = self._cloudsources_client
+    else:
+      client = self._client
+    return client
+
   def _FindSourceMethod(self, source_crd, method_name):
     """Returns the given method for the given source kind.
 
@@ -359,11 +405,15 @@ class AnthosEventsOperations(object):
     Returns:
       registry.APIMethod, holds information for the requested method.
     """
+    if 'v1' in source_crd.getActiveSourceVersions():
+      source_api_version = 'v1'
+    else:
+      source_api_version = 'v1beta1'
     return registry.GetMethod(
         util.ANTHOS_SOURCE_COLLECTION_NAME.format(
             plural_kind=source_crd.source_kind_plural),
         method_name,
-        api_version=self._api_version)
+        api_version=source_api_version)
 
   def SourceGetMethod(self, source_crd):
     """Returns the request method for a Get request of this source."""
@@ -379,15 +429,18 @@ class AnthosEventsOperations(object):
 
   def GetSource(self, source_ref, source_crd):
     """Returns the referenced source."""
+    client = self.ClientFromSourceCrd(source_crd)
+    messages = client.MESSAGES_MODULE
+
     request_method = self.SourceGetMethod(source_crd)
     request_message_type = request_method.GetRequestType()
     request = request_message_type(name=source_ref.RelativeName())
     try:
       with metrics.RecordDuration(metric_names.GET_SOURCE):
-        response = request_method.Call(request, client=self._client)
+        response = request_method.Call(request, client=client)
     except api_exceptions.HttpNotFoundError:
       return None
-    return source.Source(response, self.messages, source_crd.source_kind)
+    return source.Source(response, messages, source_crd.source_kind)
 
   def CreateSource(self, source_obj, source_crd, owner_trigger, namespace_ref,
                    broker_name, parameters):
@@ -406,17 +459,24 @@ class AnthosEventsOperations(object):
     Returns:
       source.Source of the created source.
     """
+    client = self.ClientFromSourceCrd(source_crd)
+    messages = client.MESSAGES_MODULE
+
     source_obj.ce_overrides[trigger.SOURCE_TRIGGER_LINK_FIELD] = (
         owner_trigger.filter_attributes[trigger.SOURCE_TRIGGER_LINK_FIELD])
+
     source_obj.owners.append(
-        self.messages.OwnerReference(
+        messages.OwnerReference(
             apiVersion=owner_trigger.apiVersion,
             kind=owner_trigger.kind,
             name=owner_trigger.name,
             uid=owner_trigger.uid,
             controller=True))
     source_obj.set_sink(broker_name, self._api_version)
-    arg_utils.ParseStaticFieldsIntoMessage(source_obj.spec, parameters)
+
+    # Parse parameters flags into source's spec
+    source.ParseDynamicFieldsIntoMessage(source_obj.spec, parameters)
+    source.SourceFix(source_obj)
 
     request_method = self.SourceCreateMethod(source_crd)
     request_message_type = request_method.GetRequestType()
@@ -425,19 +485,19 @@ class AnthosEventsOperations(object):
         'parent': namespace_ref.RelativeName()})
     try:
       with metrics.RecordDuration(metric_names.CREATE_SOURCE):
-        response = request_method.Call(request, client=self._client)
+        response = request_method.Call(request, client=client)
     except api_exceptions.HttpConflictError:
       raise exceptions.SourceCreationError(
           'Source [{}] already exists.'.format(source_obj.name))
 
-    return source.Source(response, self.messages, source_crd.source_kind)
+    return source.Source(response, messages, source_crd.source_kind)
 
-  def PollSource(self, source_obj, event_type, tracker):
+  def PollSource(self, source_obj, event_type_obj, tracker):
     """Wait for source to be Ready == True."""
     source_ref = util.GetSourceRef(source_obj.name, source_obj.namespace,
-                                   event_type.crd, True)
-    source_getter = functools.partial(
-        self.GetSource, source_ref, event_type.crd)
+                                   event_type_obj.crd, True)
+    source_getter = functools.partial(self.GetSource, source_ref,
+                                      event_type_obj.crd)
     poller = SourceConditionPoller(source_getter, tracker,
                                    stages.TriggerSourceDependencies())
     util.WaitForCondition(poller, exceptions.SourceCreationError)
@@ -447,17 +507,28 @@ class AnthosEventsOperations(object):
 
   def DeleteSource(self, source_ref, source_crd):
     """Deletes the referenced source."""
+    client = self.ClientFromSourceCrd(source_crd)
+
     request_method = self.SourceDeleteMethod(source_crd)
     request_message_type = request_method.GetRequestType()
     request = request_message_type(name=source_ref.RelativeName())
     try:
       with metrics.RecordDuration(metric_names.DELETE_SOURCE):
-        request_method.Call(request, client=self._client)
+        request_method.Call(request, client=client)
     except api_exceptions.HttpNotFoundError:
       raise exceptions.SourceNotFound(
           '{} events source [{}] not found.'.format(
               source_crd.source_kind, source_ref.Name()))
 
+  def GetCustomResourceDefinition(self, crd_name):
+    """Returns customresourcedefinition."""
+    messages = self._crd_client.MESSAGES_MODULE
+    request = messages.AnthoseventsCustomresourcedefinitionsGetRequest(
+        name='customresourcedefinitions/{}'.format(crd_name))
+    response = self._crd_client.customresourcedefinitions.Get(request)
+    return custom_resource_definition.CustomResourceDefinition(
+        response, messages)
+
   def ListSourceCustomResourceDefinitions(self):
     """Returns a list of CRDs for event sources."""
     # Passing the parent field is only needed for hosted, but shouldn't hurt
@@ -477,6 +548,12 @@ class AnthosEventsOperations(object):
         custom_resource_definition.SourceCustomResourceDefinition(
             item, messages) for item in response.items
     ]
+
+    # The customresourcedefinition received in listResponse is missing their
+    # apiVersion's (intended), so manually insert apiVersion from listResponse.
+    for source_crd in source_crds:
+      source_crd.setApiVersion(response.apiVersion)
+
     # Only include CRDs for source kinds that are defined in the api.
     return [s for s in source_crds if hasattr(self.messages, s.source_kind)]
 
