@@ -22,6 +22,7 @@ import argparse
 import functools
 import itertools
 import sys
+import textwrap
 
 from googlecloudsdk.api_lib.ml_engine import jobs
 from googlecloudsdk.api_lib.ml_engine import versions_api
@@ -31,6 +32,9 @@ from googlecloudsdk.calliope import arg_parsers
 from googlecloudsdk.calliope import base
 from googlecloudsdk.calliope.concepts import concepts
 from googlecloudsdk.command_lib.iam import completers as iam_completers
+from googlecloudsdk.command_lib.iam import iam_util as core_iam_util
+from googlecloudsdk.command_lib.kms import resource_args as kms_resource_args
+from googlecloudsdk.command_lib.ml_engine import constants
 from googlecloudsdk.command_lib.ml_engine import models_util
 from googlecloudsdk.command_lib.util.apis import arg_utils
 from googlecloudsdk.command_lib.util.concepts import concept_parsers
@@ -121,7 +125,7 @@ def GetDescriptionFlag(noun):
       '--description',
       required=False,
       default=None,
-      help='The description of the {noun}.'.format(noun=noun))
+      help='Description of the {noun}.'.format(noun=noun))
 
 # Run flags
 DISTRIBUTED = base.Argument(
@@ -140,6 +144,11 @@ WORKERS = base.Argument(
     type=int,
     help=('Number of workers with which to run. '
           'Ignored if --distributed is not specified. Default: 2'))
+EVALUATORS = base.Argument(
+    '--evaluator-count',
+    type=int,
+    help=('Number of evaluators with which to run. '
+          'Ignored if --distributed is not specified. Default: 0'))
 START_PORT = base.Argument(
     '--start-port',
     type=int,
@@ -201,11 +210,11 @@ JOB_NAME = base.Argument('job', help='Name of the job.')
 PACKAGE_PATH = base.Argument(
     '--package-path',
     help="""\
-Path to a Python package to build. This should point to a directory containing
-the Python source for the job. It will be built using *setuptools* (which must be
-installed) using its *parent* directory as context. If the parent directory
-contains a `setup.py` file, the build will use that; otherwise, it will use a
-simple built-in one.
+Path to a Python package to build. This should point to a *local* directory
+containing the Python source for the job. It will be built using *setuptools*
+(which must be installed) using its *parent* directory as context. If the parent
+directory contains a `setup.py` file, the build will use that; otherwise,
+it will use a simple built-in one.
 """)
 PACKAGES = base.Argument(
     '--packages',
@@ -219,12 +228,63 @@ Storage bucket given by `--staging-bucket`, or Cloud Storage URLs
 ('gs://bucket-name/path/to/package.tar.gz').
 """)
 
+_REGION_FLAG_HELPTEXT = """\
+Google Cloud region of the regional endpoint to use for this command.
+If unspecified, the command uses the global endpoint of the AI Platform Training
+and Prediction API.
+
+Learn more about regional endpoints and see a list of available regions:
+ https://cloud.google.com/ai-platform/prediction/docs/regional-endpoints
+"""
+
+
+def GetRegionArg(include_global=False):
+  """Adds --region flag to determine endpoint for models and versions."""
+  if include_global:
+    return base.Argument(
+        '--region',
+        choices=constants.SUPPORTED_REGIONS_WITH_GLOBAL,
+        help=_REGION_FLAG_HELPTEXT)
+  return base.Argument(
+      '--region',
+      choices=constants.SUPPORTED_REGIONS,
+      help=_REGION_FLAG_HELPTEXT)
+
 
 SERVICE_ACCOUNT = base.Argument(
     '--service-account',
     required=False,
     help='Specifies the service account for resource access control.')
 
+NETWORK = base.Argument(
+    '--network',
+    help="""\
+Full name of the Google Compute Engine
+network (https://cloud.google.com/vpc/docs) to which the Job
+is peered with. For example, ``projects/12345/global/networks/myVPC''. The
+format is of the form projects/{project}/global/networks/{network}, where
+{project} is a project number, as in '12345', and {network} is network name.
+Private services access must already have been configured
+(https://cloud.google.com/vpc/docs/configure-private-services-access)
+for the network. If unspecified, the Job is not peered with any network.
+""")
+
+TRAINING_SERVICE_ACCOUNT = base.Argument(
+    '--service-account',
+    type=core_iam_util.GetIamAccountFormatValidator(),
+    required=False,
+    help=textwrap.dedent("""\
+      The email address of a service account to use when running the
+      training appplication. You must have the `iam.serviceAccounts.actAs`
+      permission for the specified service account. In addition, the AI Platform
+      Training Google-managed service account must have the
+      `roles/iam.serviceAccountAdmin` role for the specified service account.
+      [Learn more about configuring a service
+      account.](/ai-platform/training/docs/custom-service-account)
+      If not specified, the AI Platform Training Google-managed service account
+      is used by default.
+      """))
+
 
 def GetModuleNameFlag(required=True):
   return base.Argument(
@@ -253,11 +313,12 @@ def GetJobDirFlag(upload_help=True, allow_local=False):
 {dir_type} in which to store training outputs and other data
 needed for training.
 
-This path will be passed to your TensorFlow program as the `--job_dir` command-line
+This path will be passed to your TensorFlow program as the `--job-dir` command-line
 arg. The benefit of specifying this field is that AI Platform will validate
-the path for use in training.
+the path for use in training. However, note that your training program will need
+to parse the provided `--job-dir` argument.
 """.format(
-    dir_type=('Google Cloud Storage path' +
+    dir_type=('Cloud Storage path' +
               (' or local_directory' if allow_local else '')))
   if upload_help:
     help_ += """\
@@ -293,10 +354,10 @@ VERSION_NAME = base.Argument('version', help='Name of the model version.')
 RUNTIME_VERSION = base.Argument(
     '--runtime-version',
     help=(
-        'AI Platform runtime version for this job. Defaults '
-        'to a stable version, which is defined in documentation along '
-        'with the list of supported versions: '
-        'https://cloud.google.com/ml-engine/docs/tensorflow/runtime-version-list'  # pylint: disable=line-too-long
+        'AI Platform runtime version for this job. Must be specified unless '
+        '--master-image-uri is specified instead. It is defined in '
+        'documentation along with the list of supported versions: '
+        'https://cloud.google.com/ai-platform/prediction/docs/runtime-version-list'  # pylint: disable=line-too-long
     ))
 
 
@@ -329,16 +390,30 @@ FRAMEWORK_MAPPER = arg_utils.ChoiceEnumMapper(
     (versions_api.GetMessagesModule().
      GoogleCloudMlV1Version.FrameworkValueValuesEnum),
     custom_mappings=_FRAMEWORK_CHOICES,
-    help_str=('The ML framework used to train this version of the model. '
+    help_str=('ML framework used to train this version of the model. '
               'If not specified, defaults to \'tensorflow\''))
 
 
+def AddKmsKeyFlag(parser, resource):
+  permission_info = '{} must hold permission {}'.format(
+      "The 'AI Platform Service Agent' service account",
+      "'Cloud KMS CryptoKey Encrypter/Decrypter'")
+  kms_resource_args.AddKmsKeyResourceArg(
+      parser, resource, permission_info=permission_info)
+
+
 def AddPythonVersionFlag(parser, context):
-  help_str = (
-      'Version of Python used {context}. If not set, the default '
-      'version is 2.7. Python 3.5 is available when `--runtime-version` is '
-      'set to 1.4 and above. Python 2.7 works with all supported runtime '
-      'versions.').format(context=context)
+  help_str = """\
+      Version of Python used {context}. Choices are 3.7, 3.5, and 2.7.
+      However, this value must be compatible with the chosen runtime version
+      for the job.
+
+      Must be used with a compatible runtime version:
+
+      * 3.7 is compatible with runtime versions 1.15 and later.
+      * 3.5 is compatible with runtime versions 1.4 through 1.14.
+      * 2.7 is compatible with runtime versions 1.15 and earlier.
+      """.format(context=context)
   version = base.Argument(
       '--python-version',
       help=help_str)
@@ -381,9 +456,9 @@ SIGNATURE_NAME = base.Argument(
     required=False,
     type=str,
     help="""\
-    The name of the signature defined in the SavedModel to use for
+    Name of the signature defined in the SavedModel to use for
     this job. Defaults to DEFAULT_SERVING_SIGNATURE_DEF_KEY in
-    https://www.tensorflow.org/api_docs/python/tf/saved_model/signature_constants,
+    https://www.tensorflow.org/api_docs/python/tf/compat/v1/saved_model/signature_constants,
     which is "serving_default". Only applies to TensorFlow models.
     """)
 
@@ -440,13 +515,13 @@ def GetPredictJobSummary():
 def ModelAttributeConfig():
   return concepts.ResourceParameterAttributeConfig(
       name='model',
-      help_text='The model for the {resource}.')
+      help_text='Model for the {resource}.')
 
 
 def VersionAttributeConfig():
   return concepts.ResourceParameterAttributeConfig(
       name='version',
-      help_text='The version for the {resource}.')
+      help_text='Version for the {resource}.')
 
 
 def GetVersionResourceSpec():
@@ -467,6 +542,59 @@ def AddVersionResourceArg(parser, verb):
       required=True).AddToParser(parser)
 
 
+def GetModelResourceSpec(resource_name='model'):
+  return concepts.ResourceSpec(
+      'ml.projects.models',
+      resource_name=resource_name,
+      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
+      disable_auto_completers=True)
+
+
+def GetModelResourceArg(positional=True, required=False, verb=''):
+  """Add a resource argument for AI Platform model.
+
+  NOTE: Must be used only if it's the only resource arg in the command.
+
+  Args:
+    positional: bool, if True, means that the model is a positional rather
+    required: bool, if True means that argument is required.
+    verb: str, the verb to describe the resource, such as 'to update'.
+
+  Returns:
+    An argparse.ArgumentParse object.
+  """
+  if positional:
+    name = 'model'
+  else:
+    name = '--model'
+  return concept_parsers.ConceptParser.ForResource(
+      name,
+      GetModelResourceSpec(),
+      'The AI Platform model {}.'.format(verb),
+      required=required)
+
+
+def GetLocationResourceSpec():
+  return concepts.ResourceSpec(
+      'ml.projects.locations',
+      resource_name='location',
+      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
+      disable_auto_completers=True)
+
+
+def GetLocationResourceArg():
+  """Add a resource argument for AI Platform location.
+
+  Returns:
+    An argparse.ArgumentParse object.
+  """
+  return concept_parsers.ConceptParser.ForResource(
+      'location',
+      GetLocationResourceSpec(),
+      'The location you want to show details for.',
+      required=True)
+
+
 def AddUserCodeArgs(parser):
   """Add args that configure user prediction code."""
   user_code_group = base.ArgumentGroup(help="""\
@@ -478,7 +606,7 @@ def AddUserCodeArgs(parser):
   user_code_group.AddArgument(base.Argument(
       '--prediction-class',
       help="""\
-          The fully-qualified name of the custom prediction class in the package
+          Fully-qualified name of the custom prediction class in the package
           provided for custom prediction.
 
           For example, `--prediction-class=my_package.SequenceModel`.
@@ -488,7 +616,7 @@ def AddUserCodeArgs(parser):
       type=arg_parsers.ArgList(),
       metavar='PACKAGE_URI',
       help="""\
-          Comma-separated list of Google Cloud Storage URIs ('gs://...') for
+          Comma-separated list of Cloud Storage URIs ('gs://...') for
           user-supplied Python packages to use.
           """))
   user_code_group.AddToParser(parser)
@@ -501,17 +629,19 @@ def GetAcceleratorFlag():
           spec={
               'type': str,
               'count': int,
-          }, required_keys=['type', 'count']),
+          }, required_keys=['type']),
       help="""\
-Manage the accelerator config for GPU serving. When deploying a model with the
-new Alpha Google Compute Engine Machine Types, a GPU accelerator may also
-be selected. Accelerator config for version creation is currently available
-in us-central1 only.
+Manage the accelerator config for GPU serving. When deploying a model with
+Compute Engine Machine Types, a GPU accelerator may also
+be selected.
 
 *type*::: The type of the accelerator. Choices are {}.
 
 *count*::: The number of accelerators to attach to each machine running the job.
-""".format(', '.join(
+ If not specified, the default value is 1. Your model must be specially designed
+to accommodate more than 1 accelerator per machine. To configure how many
+replicas your model has, set the `manualScaling` or `autoScaling`
+parameters.""".format(', '.join(
     ["'{}'".format(c) for c in _OP_ACCELERATOR_TYPE_MAPPER.choices])))
 
 
@@ -525,7 +655,7 @@ def ParseAcceleratorFlag(accelerator):
     raise ArgumentError("""\
 The type of the accelerator can only be one of the following: {}.
 """.format(', '.join(["'{}'".format(c) for c in types])))
-  accelerator_count = accelerator.get('count', 0)
+  accelerator_count = accelerator.get('count', 1)
   if accelerator_count <= 0:
     raise ArgumentError("""\
 The count of the accelerator must be greater than 0.
@@ -543,7 +673,7 @@ def AddExplainabilityFlags(parser):
   """Add args that configure explainability."""
   base.ChoiceArgument(
       '--explanation-method',
-      choices=['integrated-gradients', 'sampled-shapley'],
+      choices=['integrated-gradients', 'sampled-shapley', 'xrai'],
       required=False,
       help_str="""\
           Enable explanations and select the explanation method to use.
@@ -551,6 +681,7 @@ def AddExplainabilityFlags(parser):
           The valid options are:
             integrated-gradients: Use Integrated Gradients.
             sampled-shapley: Use Sampled Shapley.
+            xrai: Use XRAI.
       """
   ).AddToParser(parser)
   base.Argument(
@@ -560,7 +691,8 @@ def AddExplainabilityFlags(parser):
       required=False,
       help="""\
           Number of integral steps for Integrated Gradients. Only valid when
-          `--explanation-method=integrated-gradients` is specified.
+          `--explanation-method=integrated-gradients` or
+          `--explanation-method=xrai` is specified.
       """
   ).AddToParser(parser)
   base.Argument(
@@ -586,6 +718,7 @@ def AddCustomContainerFlags(parser, support_tpu_tf_version=False):
   GetWorkerMachineConfig().AddToParser(parser)
   GetWorkerAccelerator().AddToParser(parser)
   GetWorkerImageUri().AddToParser(parser)
+  GetUseChiefInTfConfig().AddToParser(parser)
   if support_tpu_tf_version:
     GetTpuTfVersion().AddToParser(parser)
 
@@ -594,7 +727,7 @@ _ACCELERATOR_TYPE_MAPPER = arg_utils.ChoiceEnumMapper(
     'generic-accelerator',
     jobs.GetMessagesModule(
     ).GoogleCloudMlV1AcceleratorConfig.TypeValueValuesEnum,
-    help_str='The available types of accelerators.',
+    help_str='Available types of accelerators.',
     include_filter=lambda x: x != 'ACCELERATOR_TYPE_UNSPECIFIED',
     required=False)
 
@@ -602,10 +735,18 @@ _OP_ACCELERATOR_TYPE_MAPPER = arg_utils.ChoiceEnumMapper(
     'generic-accelerator',
     jobs.GetMessagesModule().GoogleCloudMlV1AcceleratorConfig
     .TypeValueValuesEnum,
-    help_str='The available types of accelerators.',
+    help_str='Available types of accelerators.',
     include_filter=lambda x: x.startswith('NVIDIA'),
     required=False)
 
+_OP_AUTOSCALING_METRIC_NAME_MAPPER = arg_utils.ChoiceEnumMapper(
+    'autoscaling-metric-name',
+    versions_api.GetMessagesModule().GoogleCloudMlV1MetricSpec
+    .NameValueValuesEnum,
+    help_str='The available metric names.',
+    include_filter=lambda x: x != 'METRIC_NAME_UNSPECIFIED',
+    required=False)
+
 _ACCELERATOR_TYPE_HELP = """\
    Hardware accelerator config for the {worker_type}. Must specify
    both the accelerator type (TYPE) for each server and the number of
@@ -627,11 +768,35 @@ def _ValidateAcceleratorCount(accelerator_count):
   return count
 
 
+def _ValidateMetricTargetKey(key):
+  """Value validation for Metric target name."""
+  names = list(_OP_AUTOSCALING_METRIC_NAME_MAPPER.choices)
+  if key not in names:
+    raise ArgumentError("""\
+The autoscaling metric name can only be one of the following: {}.
+""".format(', '.join(["'{}'".format(c) for c in names])))
+  return key
+
+
+def _ValidateMetricTargetValue(value):
+  """Value validation for Metric target value."""
+  try:
+    result = int(value)
+  except (TypeError, ValueError):
+    raise ArgumentError('Metric target percentage value %s is not an integer.' %
+                        value)
+
+  if result < 0 or result > 100:
+    raise ArgumentError(
+        'Metric target value %s is not between 0 and 100.' % value)
+  return result
+
+
 def _MakeAcceleratorArgConfigArg(arg_name, arg_help, required=False):
   """Creates an ArgDict representing an AcceleratorConfig message."""
-  type_help = '*type*::: The type of the accelerator. Choices are {}'.format(
+  type_help = '*type*::: Type of the accelerator. Choices are {}'.format(
       ','.join(_ACCELERATOR_TYPE_MAPPER.choices))
-  count_help = ('*count*::: The number of accelerators to attach to each '
+  count_help = ('*count*::: Number of accelerators to attach to each '
                 'machine running the job. Must be greater than 0.')
   arg = base.Argument(
       arg_name,
@@ -675,7 +840,7 @@ def GetMasterImageUri():
       '--master-image-uri',
       required=False,
       help=('Docker image to run on each master worker. '
-            'This image must be in Google Container Registry. Only one of '
+            'This image must be in Container Registry. Only one of '
             '`--master-image-uri` and `--runtime-version` must be specified.'))
 
 
@@ -693,7 +858,7 @@ def GetParameterServerMachineTypeConfig():
       '--parameter-server-count',
       type=arg_parsers.BoundedInt(1, sys.maxsize, unlimited=True),
       required=True,
-      help=('The number of parameter servers to use for the training job.'))
+      help=('Number of parameter servers to use for the training job.'))
 
   machine_type_group = base.ArgumentGroup(
       required=False,
@@ -716,7 +881,7 @@ def GetParameterServerImageUri():
       '--parameter-server-image-uri',
       required=False,
       help=('Docker image to run on each parameter server. '
-            'This image must be in Google Container Registry. If not '
+            'This image must be in Container Registry. If not '
             'specified, the value of `--master-image-uri` is used.'))
 
 
@@ -733,7 +898,7 @@ def GetWorkerMachineConfig():
       '--worker-count',
       type=arg_parsers.BoundedInt(1, sys.maxsize, unlimited=True),
       required=True,
-      help='The number of worker nodes to use for the training job.')
+      help='Number of worker nodes to use for the training job.')
 
   machine_type_group = base.ArgumentGroup(
       required=False,
@@ -756,7 +921,7 @@ def GetWorkerImageUri():
       '--worker-image-uri',
       required=False,
       help=('Docker image to run on each worker node. '
-            'This image must be in Google Container Registry. If not '
+            'This image must be in Container Registry. If not '
             'specified, the value of `--master-image-uri` is used.'))
 
 
@@ -770,10 +935,108 @@ def GetTpuTfVersion():
             'being used.'))
 
 
+def GetUseChiefInTfConfig():
+  """Build use-chief-in-tf-config flag."""
+  return base.Argument(
+      '--use-chief-in-tf-config',
+      required=False,
+      type=arg_parsers.ArgBoolean(),
+      help=('Use "chief" role in the cluster instead of "master". This is '
+            'required for TensorFlow 2.0 and newer versions. Unlike "master" '
+            'node, "chief" node does not run evaluation.'))
+
+
 def AddMachineTypeFlagToParser(parser):
   base.Argument(
       '--machine-type',
       help="""\
 Type of machine on which to serve the model. Currently only applies to online prediction. For available machine types,
-see https://cloud.google.com/ml-engine/docs/tensorflow/online-predict#machine-types.
+see https://cloud.google.com/ai-platform/prediction/docs/machine-types-online-prediction#available_machine_types.
 """).AddToParser(parser)
+
+
+def AddContainerFlags(parser):
+  """Adds flags related to custom containers to the specified parser."""
+  container_group = parser.add_argument_group(
+      help='Configure the container to be deployed.')
+  container_group.add_argument(
+      '--image',
+      help="""\
+Name of the container image to deploy (e.g. gcr.io/myproject/server:latest).
+""")
+  container_group.add_argument(
+      '--command',
+      type=arg_parsers.ArgList(),
+      metavar='COMMAND',
+      action=arg_parsers.UpdateAction,
+      help="""\
+Entrypoint for the container image. If not specified, the container
+image's default Entrypoint is run.
+""")
+  container_group.add_argument(
+      '--args',
+      metavar='ARG',
+      type=arg_parsers.ArgList(),
+      action=arg_parsers.UpdateAction,
+      help="""\
+Comma-separated arguments passed to the command run by the container
+image. If not specified and no '--command' is provided, the container
+image's default Cmd is used.
+""")
+  container_group.add_argument(
+      '--env-vars',
+      metavar='KEY=VALUE',
+      type=arg_parsers.ArgDict(),
+      action=arg_parsers.UpdateAction,
+      help='List of key-value pairs to set as environment variables.'
+  )
+  container_group.add_argument(
+      '--ports',
+      metavar='ARG',
+      type=arg_parsers.ArgList(element_type=arg_parsers.BoundedInt(1, 65535)),
+      action=arg_parsers.UpdateAction,
+      help="""\
+Container ports to receive requests at. Must be a number between 1 and 65535,
+inclusive.
+""")
+  route_group = parser.add_argument_group(
+      help='Flags to control the paths that requests and health checks are '
+      'sent to.')
+  route_group.add_argument(
+      '--predict-route',
+      help='HTTP path to send prediction requests to inside the container.'
+  )
+  route_group.add_argument(
+      '--health-route',
+      help='HTTP path to send health checks to inside the container.'
+  )
+
+
+def AddAutoScalingFlags(parser):
+  """Adds flags related to autoscaling to the specified parser."""
+  autoscaling_group = parser.add_argument_group(
+      help='Configure the autoscaling settings to be deployed.')
+  autoscaling_group.add_argument(
+      '--min-nodes',
+      type=int,
+      help="""\
+The minimum number of nodes to scale this model under load.
+""")
+  autoscaling_group.add_argument(
+      '--max-nodes',
+      type=int,
+      help="""\
+The maximum number of nodes to scale this model under load.
+""")
+  autoscaling_group.add_argument(
+      '--metric-targets',
+      metavar='METRIC-NAME=TARGET',
+      type=arg_parsers.ArgDict(
+          key_type=_ValidateMetricTargetKey, value_type=_ValidateMetricTargetValue),
+      action=arg_parsers.UpdateAction,
+      default={},
+      help="""\
+List of key-value pairs to set as metrics' target for autoscaling.
+Autoscaling could be based on CPU usage or GPU duty cycle, valid key could be
+cpu-usage or gpu-duty-cycle.
+""")
