@@ -165,6 +165,8 @@ def _AddMutuallyExclusiveArgs(mutex_group, release_track):
                 api_adapter.NETWORK_POLICY: _ParseAddonDisabled,
                 api_adapter.CLOUDRUN: _ParseAddonDisabled,
                 api_adapter.NODELOCALDNS: _ParseAddonDisabled,
+                api_adapter.CONFIGCONNECTOR: _ParseAddonDisabled,
+                api_adapter.GCEPDCSIDRIVER: _ParseAddonDisabled,
             }),
         dest='disable_addons',
         metavar='ADDON=ENABLED|DISABLED',
@@ -174,13 +176,17 @@ def _AddMutuallyExclusiveArgs(mutex_group, release_track):
 {dashboard}=ENABLED|DISABLED
 {network_policy}=ENABLED|DISABLED
 {cloudrun}=ENABLED|DISABLED
-{nodelocaldns}=ENABLED|DISABLED""".format(
+{configconnector}=ENABLED|DISABLED
+{nodelocaldns}=ENABLED|DISABLED
+{gcepdcsidriver}=ENABLED|DISABLED""".format(
     hpa=api_adapter.HPA,
     ingress=api_adapter.INGRESS,
     dashboard=api_adapter.DASHBOARD,
     network_policy=api_adapter.NETWORK_POLICY,
     cloudrun=api_adapter.CLOUDRUN,
+    configconnector=api_adapter.CONFIGCONNECTOR,
     nodelocaldns=api_adapter.NODELOCALDNS,
+    gcepdcsidriver=api_adapter.GCEPDCSIDRIVER,
     ))
 
   mutex_group.add_argument(
@@ -284,23 +290,28 @@ class Update(base.UpdateCommand):
     flags.AddWorkloadIdentityUpdateFlags(group)
     flags.AddDatabaseEncryptionFlag(group)
     flags.AddDisableDatabaseEncryptionFlag(group)
+    flags.AddDisableDefaultSnatFlag(group, for_cluster_create=False)
     flags.AddVerticalPodAutoscalingFlag(group)
-    flags.AddAutoprovisioningFlags(group, ga=True)
+    flags.AddAutoprovisioningFlags(group)
     flags.AddEnableShieldedNodesFlags(group)
+    flags.AddMasterGlobalAccessFlag(group, is_update=True)
+    flags.AddPrivateIpv6GoogleAccessTypeFlag('v1', group, hidden=False)
 
   def ParseUpdateOptions(self, args, locations):
     opts = container_command_util.ParseUpdateOptionsBase(args, locations)
     opts.resource_usage_bigquery_dataset = args.resource_usage_bigquery_dataset
     opts.clear_resource_usage_bigquery_dataset = \
         args.clear_resource_usage_bigquery_dataset
-    opts.cloud_run_config = args.cloud_run_config
     opts.enable_network_egress_metering = args.enable_network_egress_metering
     opts.enable_resource_consumption_metering = \
         args.enable_resource_consumption_metering
     opts.enable_intra_node_visibility = args.enable_intra_node_visibility
+    opts.enable_master_global_access = args.enable_master_global_access
     opts.enable_shielded_nodes = args.enable_shielded_nodes
     opts.release_channel = args.release_channel
-    flags.ValidateCloudRunConfigUpdateArgs(args.cloud_run_config, args.disable_addons)
+    flags.ValidateCloudRunConfigUpdateArgs(args.cloud_run_config,
+                                           args.disable_addons)
+    opts.cloud_run_config = args.cloud_run_config
     if args.disable_addons and api_adapter.NODELOCALDNS in args.disable_addons:
       # NodeLocalDNS is being enabled or disabled
       console_io.PromptContinue(
@@ -310,6 +321,7 @@ class Update(base.UpdateCommand):
           'operations on the cluster (including delete) until it has run '
           'to completion.',
           cancel_on_no=True)
+    opts.disable_default_snat = args.disable_default_snat
     return opts
 
   def Run(self, args):
@@ -359,6 +371,7 @@ class Update(base.UpdateCommand):
     if hasattr(args, 'node_locations') and args.node_locations is not None:
       locations = sorted(args.node_locations)
 
+    flags.LogBasicAuthDeprecationWarning(args)
     if args.IsSpecified('username') or args.IsSpecified('enable_basic_auth'):
       flags.MungeBasicAuthFlags(args)
       options = api_adapter.SetMasterAuthOptions(
@@ -566,6 +579,7 @@ class UpdateBeta(Update):
     flags.AddMonitoringServiceFlag(group_logging_monitoring)
     flags.AddEnableStackdriverKubernetesFlag(group)
     flags.AddEnableLoggingMonitoringSystemOnlyFlag(group)
+    flags.AddEnableWorkloadMonitoringEapFlag(group)
     flags.AddEnableMasterSignalsFlags(group)
     flags.AddMasterAuthorizedNetworksFlags(
         parser, enable_group_for_update=group)
@@ -588,18 +602,23 @@ class UpdateBeta(Update):
     flags.AddIstioConfigFlag(parser)
     flags.AddCloudRunConfigFlag(parser)
     flags.AddEnableIntraNodeVisibilityFlag(group)
-    flags.AddWorkloadIdentityFlags(group, use_workload_pool=False)
-    flags.AddWorkloadIdentityUpdateFlags(group)
+    flags.AddWorkloadIdentityFlags(
+        group, use_identity_provider=True, use_workload_certificates=True)
+    flags.AddWorkloadIdentityUpdateFlags(group, use_workload_certificates=True)
+    flags.AddGkeOidcFlag(group)
     flags.AddDatabaseEncryptionFlag(group)
     flags.AddDisableDatabaseEncryptionFlag(group)
     flags.AddReleaseChannelFlag(group, is_update=True, hidden=False)
     flags.AddEnableShieldedNodesFlags(group)
     flags.AddTpuFlags(group, enable_tpu_service_networking=True)
-    flags.AddMasterGlobalAccessFlag(group)
+    flags.AddMasterGlobalAccessFlag(group, is_update=True)
     flags.AddEnableGvnicFlag(group)
     flags.AddDisableDefaultSnatFlag(group, for_cluster_create=False)
-    flags.AddNotificationConfigFlag(group, hidden=True)
-    flags.AddPrivateIpv6GoogleAccessTypeFlag('v1beta1', group, hidden=True)
+    flags.AddNotificationConfigFlag(group)
+    flags.AddPrivateIpv6GoogleAccessTypeFlag('v1beta1', group, hidden=False)
+    flags.AddKubernetesObjectsExportConfig(group)
+    flags.AddDisableAutopilotFlag(group, hidden=True)
+    flags.AddILBSubsettingFlags(group, hidden=True)
 
   def ParseUpdateOptions(self, args, locations):
     flags.ValidateNotificationConfigFlag(args)
@@ -613,8 +632,11 @@ class UpdateBeta(Update):
         args.clear_resource_usage_bigquery_dataset
     opts.enable_network_egress_metering = args.enable_network_egress_metering
     opts.enable_resource_consumption_metering = args.enable_resource_consumption_metering
+    opts.workload_identity_certificate_authority = args.workload_identity_certificate_authority
+    opts.disable_workload_identity_certificates = args.disable_workload_identity_certificates
     flags.ValidateIstioConfigUpdateArgs(args.istio_config, args.disable_addons)
-    flags.ValidateCloudRunConfigUpdateArgs(args.cloud_run_config, args.disable_addons)
+    flags.ValidateCloudRunConfigUpdateArgs(args.cloud_run_config,
+                                           args.disable_addons)
     if args.disable_addons and api_adapter.NODELOCALDNS in args.disable_addons:
       # NodeLocalDNS is being enabled or disabled
       console_io.PromptContinue(
@@ -624,16 +646,6 @@ class UpdateBeta(Update):
           'operations on the cluster (including delete) until it has run '
           'to completion.',
           cancel_on_no=True)
-    if args.disable_addons and api_adapter.GCEPDCSIDRIVER in args.disable_addons:
-      pdcsi_disabled = args.disable_addons[api_adapter.GCEPDCSIDRIVER]
-      if pdcsi_disabled:
-        # Persistent Disk CSI Driver is being disabled
-        console_io.PromptContinue(
-            message='If the Compute Engine Persistent Disk CSI Driver is '
-            'disabled, then any pods currently using PersistentVolumes owned '
-            'by the driver will fail to terminate. Any new pods that try to '
-            'use those PersistentVolumes will also fail to start.',
-            cancel_on_no=True)
 
     opts.enable_stackdriver_kubernetes = args.enable_stackdriver_kubernetes
     opts.enable_logging_monitoring_system_only = args.enable_logging_monitoring_system_only
@@ -645,7 +657,7 @@ class UpdateBeta(Update):
 
     # Top-level update options are automatically forced to be
     # mutually-exclusive, so we don't need special handling for these two.
-    opts.identity_namespace = args.identity_namespace
+    opts.identity_provider = args.identity_provider
     opts.enable_shielded_nodes = args.enable_shielded_nodes
     opts.enable_tpu = args.enable_tpu
     opts.tpu_ipv4_cidr = args.tpu_ipv4_cidr
@@ -654,7 +666,12 @@ class UpdateBeta(Update):
     opts.enable_gvnic = args.enable_gvnic
     opts.disable_default_snat = args.disable_default_snat
     opts.notification_config = args.notification_config
-    opts.private_ipv6_google_access_type = args.private_ipv6_google_access_type
+    opts.kubernetes_objects_changes_target = args.kubernetes_objects_changes_target
+    opts.kubernetes_objects_snapshots_target = args.kubernetes_objects_snapshots_target
+    opts.enable_gke_oidc = args.enable_gke_oidc
+    opts.enable_workload_monitoring_eap = args.enable_workload_monitoring_eap
+    opts.disable_autopilot = args.disable_autopilot
+    opts.enable_l4_ilb_subsetting = args.enable_l4_ilb_subsetting
 
     return opts
 
@@ -677,6 +694,7 @@ class UpdateAlpha(Update):
     flags.AddMonitoringServiceFlag(group_logging_monitoring)
     flags.AddEnableStackdriverKubernetesFlag(group)
     flags.AddEnableLoggingMonitoringSystemOnlyFlag(group)
+    flags.AddEnableWorkloadMonitoringEapFlag(group)
     flags.AddEnableMasterSignalsFlags(group)
     flags.AddMasterAuthorizedNetworksFlags(
         parser, enable_group_for_update=group)
@@ -700,8 +718,10 @@ class UpdateAlpha(Update):
     flags.AddIstioConfigFlag(parser)
     flags.AddCloudRunConfigFlag(parser)
     flags.AddEnableIntraNodeVisibilityFlag(group)
-    flags.AddWorkloadIdentityFlags(group, use_workload_pool=False)
-    flags.AddWorkloadIdentityUpdateFlags(group)
+    flags.AddWorkloadIdentityFlags(
+        group, use_identity_provider=True, use_workload_certificates=True)
+    flags.AddWorkloadIdentityUpdateFlags(group, use_workload_certificates=True)
+    flags.AddGkeOidcFlag(group)
     flags.AddDisableDefaultSnatFlag(group, for_cluster_create=False)
     flags.AddDatabaseEncryptionFlag(group)
     flags.AddDisableDatabaseEncryptionFlag(group)
@@ -709,10 +729,13 @@ class UpdateAlpha(Update):
     flags.AddReleaseChannelFlag(group, is_update=True, hidden=False)
     flags.AddEnableShieldedNodesFlags(group)
     flags.AddTpuFlags(group, enable_tpu_service_networking=True)
-    flags.AddMasterGlobalAccessFlag(group)
+    flags.AddMasterGlobalAccessFlag(group, is_update=True)
     flags.AddEnableGvnicFlag(group)
-    flags.AddNotificationConfigFlag(group, hidden=True)
-    flags.AddPrivateIpv6GoogleAccessTypeFlag('v1alpha1', group, hidden=True)
+    flags.AddNotificationConfigFlag(group)
+    flags.AddPrivateIpv6GoogleAccessTypeFlag('v1alpha1', group, hidden=False)
+    flags.AddKubernetesObjectsExportConfig(group)
+    flags.AddDisableAutopilotFlag(group, hidden=True)
+    flags.AddILBSubsettingFlags(group, hidden=True)
 
   def ParseUpdateOptions(self, args, locations):
     flags.ValidateNotificationConfigFlag(args)
@@ -728,8 +751,11 @@ class UpdateAlpha(Update):
     opts.enable_intra_node_visibility = args.enable_intra_node_visibility
     opts.enable_network_egress_metering = args.enable_network_egress_metering
     opts.enable_resource_consumption_metering = args.enable_resource_consumption_metering
+    opts.workload_identity_certificate_authority = args.workload_identity_certificate_authority
+    opts.disable_workload_identity_certificates = args.disable_workload_identity_certificates
     flags.ValidateIstioConfigUpdateArgs(args.istio_config, args.disable_addons)
-    flags.ValidateCloudRunConfigUpdateArgs(args.cloud_run_config, args.disable_addons)
+    flags.ValidateCloudRunConfigUpdateArgs(args.cloud_run_config,
+                                           args.disable_addons)
     if args.disable_addons and api_adapter.NODELOCALDNS in args.disable_addons:
       # NodeLocalDNS is being enabled or disabled
       console_io.PromptContinue(
@@ -739,16 +765,6 @@ class UpdateAlpha(Update):
           'operations on the cluster (including delete) until it has run '
           'to completion.',
           cancel_on_no=True)
-    if args.disable_addons and api_adapter.GCEPDCSIDRIVER in args.disable_addons:
-      pdcsi_disabled = args.disable_addons[api_adapter.GCEPDCSIDRIVER]
-      if pdcsi_disabled:
-        # GCE Persistent Disk CSI Driver is being disabled
-        console_io.PromptContinue(
-            message='If the GCE Persistent Disk CSI Driver is disabled, then any '
-            'pods currently using PersistentVolumes owned by the driver '
-            'will fail to terminate. Any new pods that try to use those '
-            'PersistentVolumes will also fail to start.',
-            cancel_on_no=True)
     opts.enable_stackdriver_kubernetes = args.enable_stackdriver_kubernetes
     opts.enable_logging_monitoring_system_only = args.enable_logging_monitoring_system_only
     opts.no_master_logs = args.no_master_logs
@@ -761,13 +777,18 @@ class UpdateAlpha(Update):
 
     # Top-level update options are automatically forced to be
     # mutually-exclusive, so we don't need special handling for these two.
-    opts.identity_namespace = args.identity_namespace
+    opts.identity_provider = args.identity_provider
     opts.enable_shielded_nodes = args.enable_shielded_nodes
     opts.disable_default_snat = args.disable_default_snat
     opts.enable_cost_management = args.enable_cost_management
     opts.enable_master_global_access = args.enable_master_global_access
     opts.enable_gvnic = args.enable_gvnic
     opts.notification_config = args.notification_config
-    opts.private_ipv6_google_access_type = args.private_ipv6_google_access_type
+    opts.kubernetes_objects_changes_target = args.kubernetes_objects_changes_target
+    opts.kubernetes_objects_snapshots_target = args.kubernetes_objects_snapshots_target
+    opts.enable_gke_oidc = args.enable_gke_oidc
+    opts.enable_workload_monitoring_eap = args.enable_workload_monitoring_eap
+    opts.disable_autopilot = args.disable_autopilot
+    opts.enable_l4_ilb_subsetting = args.enable_l4_ilb_subsetting
 
     return opts
