@@ -34,7 +34,7 @@ extern "C" {
   /* The #define below is used for apps that dynamically link with nDPI to make
      sure that datastructures and in sync across versions
   */
-#define NDPI_API_VERSION                      1
+#define NDPI_API_VERSION                      9072
 
 #define SIZEOF_ID_STRUCT                      ( sizeof(struct ndpi_id_struct)   )
 #define SIZEOF_FLOW_STRUCT                    ( sizeof(struct ndpi_flow_struct) )
@@ -78,9 +78,34 @@ extern "C" {
    */
   u_int32_t ndpi_detection_get_sizeof_ndpi_id_struct(void);
 
+
+  /**
+   * Get the size of the flow tcp struct
+   *
+   * @return the size of the flow tcp struct
+   *
+   */
+  u_int32_t ndpi_detection_get_sizeof_ndpi_flow_tcp_struct(void);
+
+
+  /**
+   * Get the size of the flow udp struct
+   *
+   * @return the size of the flow udp struct
+   *
+   */
+  u_int32_t ndpi_detection_get_sizeof_ndpi_flow_udp_struct(void);
+
+  /*
+    Same as the API call above but used for matching raw id's added
+    via ndpi_add_string_value_to_automa()
+  */
+  int ndpi_match_string_value(void *_automa, char *string_to_match,
+			      u_int match_len, u_int32_t *num);
+
   /**
    * nDPI personal allocation and free functions
-  **/
+   **/
   void * ndpi_malloc(size_t size);
   void * ndpi_calloc(unsigned long count, size_t size);
   void * ndpi_realloc(void *ptr, size_t old_size, size_t new_size);
@@ -112,7 +137,7 @@ extern "C" {
    *         NULL if the substring is not found
    *
    */
-  char* ndpi_strncasestr(const char *s, const char *find, size_t slen);
+  const char* ndpi_strncasestr(const char *s, const char *find, size_t slen);
 
   /**
    * Returns the nDPI protocol id for IP-based protocol detection
@@ -126,6 +151,21 @@ extern "C" {
   u_int16_t ndpi_network_ptree_match(struct ndpi_detection_module_struct *ndpi_struct,
 				     struct in_addr *pin);
 
+  /**
+   * Returns the nDPI protocol id for IP+port-based protocol detection
+   *
+   * @par    ndpi_struct  = the struct created for the protocol detection
+   * @par    pin          = IP host address (MUST BE in network byte order):
+   *                        See man(7) ip for details
+   * @par    port         = The port (MUST BE in network byte order) or
+   *                        0 if ignored
+   * @return the nDPI protocol ID
+   *
+   */
+  u_int16_t ndpi_network_port_ptree_match(struct ndpi_detection_module_struct *ndpi_struct,
+					  struct in_addr *pin /* network byte order */,
+					  u_int16_t port /* network byte order */);
+
   /**
    * Init single protocol match
    *
@@ -142,6 +182,10 @@ extern "C" {
    * hosts and do other things. As soon as you are ready to use
    * it do not forget to call first ndpi_finalize_initalization()
    *
+   * You can call this function multiple times, (i.e. to create multiple
+   * indipendent detection contexts) but all these calls MUST NOT run
+   * in parallel
+   *
    * @par prefs = load preferences
    * @return  the initialized detection module
    *
@@ -151,7 +195,7 @@ extern "C" {
   /**
    * Completes the initialization (2nd step)
    *
-   * @return  the initialized detection module
+   * @par ndpi_str = the struct created for the protocol detection
    *
    */
   void ndpi_finalize_initalization(struct ndpi_detection_module_struct *ndpi_str);
@@ -218,7 +262,7 @@ extern "C" {
    */
   void ndpi_set_protocol_detection_bitmask2(struct ndpi_detection_module_struct *ndpi_struct,
 					    const NDPI_PROTOCOL_BITMASK * detection_bitmask);
-  
+
     /**
    *  Function to be called before we give up with detection for a given flow.
    *  This function reduces the NDPI_UNKNOWN_PROTOCOL detection
@@ -240,13 +284,13 @@ extern "C" {
    * (like SSL getting both client and server certificate even if we already know after
    * seeing the client certificate what the protocol is)
    *
-   * @par    ndpi_struct   = the detection module
-   * @par    flow          = pointer to the connection state machine
-   * @par    packet        = unsigned char pointer to the Layer 3 (IP header)
-   * @par    packetlen     = the length of the packet
-   * @par    current_tick  = the current timestamp for the packet
-   * @par    src           = pointer to the source subscriber state machine
-   * @par    dst           = pointer to the destination subscriber state machine
+   * @par    ndpi_struct    = the detection module
+   * @par    flow           = pointer to the connection state machine
+   * @par    packet         = unsigned char pointer to the Layer 3 (IP header)
+   * @par    packetlen      = the length of the packet
+   * @par    packet_time_ms = the current timestamp for the packet (expressed in msec)
+   * @par    src            = pointer to the source subscriber state machine
+   * @par    dst            = pointer to the destination subscriber state machine
    * @return void
    *
    */
@@ -254,7 +298,7 @@ extern "C" {
 				 struct ndpi_flow_struct *flow,
 				 const unsigned char *packet,
 				 const unsigned short packetlen,
-				 const u_int64_t current_tick,
+				 const u_int64_t packet_time_ms,
 				 struct ndpi_id_struct *src,
 				 struct ndpi_id_struct *dst);
 
@@ -262,13 +306,13 @@ extern "C" {
    * Processes one packet and returns the ID of the detected protocol.
    * This is the MAIN PACKET PROCESSING FUNCTION.
    *
-   * @par    ndpi_struct   = the detection module
-   * @par    flow          = pointer to the connection state machine
-   * @par    packet        = unsigned char pointer to the Layer 3 (IP header)
-   * @par    packetlen     = the length of the packet
-   * @par    current_tick  = the current timestamp for the packet
-   * @par    src           = pointer to the source subscriber state machine
-   * @par    dst           = pointer to the destination subscriber state machine
+   * @par    ndpi_struct    = the detection module
+   * @par    flow           = pointer to the connection state machine
+   * @par    packet         = unsigned char pointer to the Layer 3 (IP header)
+   * @par    packetlen      = the length of the packet
+   * @par    packet_time_ms = the current timestamp for the packet (expressed in msec)
+   * @par    src            = pointer to the source subscriber state machine
+   * @par    dst            = pointer to the destination subscriber state machine
    * @return the detected ID of the protocol
    *
    */
@@ -276,7 +320,7 @@ extern "C" {
 					      struct ndpi_flow_struct *flow,
 					      const unsigned char *packet,
 					      const unsigned short packetlen,
-					      const u_int64_t current_tick,
+					      const u_int64_t packet_time_ms,
 					      struct ndpi_id_struct *src,
 					      struct ndpi_id_struct *dst);
   /**
@@ -290,7 +334,7 @@ extern "C" {
    */
   u_int16_t ndpi_get_flow_masterprotocol(struct ndpi_detection_module_struct *ndpi_struct,
 					 struct ndpi_flow_struct *flow);
-  
+
   /**
    * API call that is called internally by ndpi_detection_process_packet or by apps
    * that want to avoid calling ndpi_detection_process_packet as they have already
@@ -300,12 +344,14 @@ extern "C" {
    * @par    ndpi_struct              = the detection module
    * @par    flow                     = the flow given for the detection module
    * @par    ndpi_selection_bitmask   = the protocol selected bitmask
+   * @return number of protocol dissector that have been tried (0 = no more dissectors)
    *
    */
-  void ndpi_check_flow_func(struct ndpi_detection_module_struct *ndpi_struct,
-			    struct ndpi_flow_struct *flow,
-			    NDPI_SELECTION_BITMASK_PROTOCOL_SIZE *ndpi_selection_packet);
+  u_int32_t ndpi_check_flow_func(struct ndpi_detection_module_struct *ndpi_struct,
+				 struct ndpi_flow_struct *flow,
+				 NDPI_SELECTION_BITMASK_PROTOCOL_SIZE *ndpi_selection_packet);
 
+  
   /**
    * Query the pointer to the layer 4 packet
    *
@@ -391,7 +437,7 @@ extern "C" {
 					char *string_to_match,
 					u_int string_to_match_len,
 					ndpi_protocol_match_result *ret_match,
-					u_int16_t master_protocol_id);  
+					u_int16_t master_protocol_id);
 
   /**
    * Check if the string content passed match with a protocol
@@ -518,7 +564,7 @@ extern "C" {
    */
   int ndpi_match_hostname_protocol(struct ndpi_detection_module_struct *ndpi_mod,
 				   struct ndpi_flow_struct *flow,
-				   u_int16_t master_protocol,				 
+				   u_int16_t master_protocol,
 				   char *name, u_int name_len);
 
   /**
@@ -742,7 +788,7 @@ extern "C" {
    * @return  0 in case of no error, or -1 if an error occurred.
    *
    */
-  int ndpi_add_string_value_to_automa(void *_automa, char *str, unsigned long num);
+  int ndpi_add_string_value_to_automa(void *_automa, char *str, u_int32_t num);
 
   /**
    * Add a string to match to an automata. Same as ndpi_add_string_value_to_automa() with num set to 1
@@ -784,12 +830,13 @@ extern "C" {
 				 u_int32_t daddr,
 				 ndpi_protocol *ret);
   int ndpi_match_custom_category(struct ndpi_detection_module_struct *ndpi_struct,
-				 char *name, u_int name_len, unsigned long *id);
+				 char *name, u_int name_len, ndpi_protocol_category_t *id);
   void ndpi_fill_protocol_category(struct ndpi_detection_module_struct *ndpi_struct,
 				   struct ndpi_flow_struct *flow,
 				   ndpi_protocol *ret);
   int ndpi_get_custom_category_match(struct ndpi_detection_module_struct *ndpi_struct,
-				     char *name_or_ip, u_int name_len, unsigned long *id);
+				     char *name_or_ip, u_int name_len,
+				     ndpi_protocol_category_t *id);
   int ndpi_set_detection_preferences(struct ndpi_detection_module_struct *ndpi_mod,
 				     ndpi_detection_preference pref,
 				     int value);
@@ -797,12 +844,13 @@ extern "C" {
   /* Tells to called on what l4 protocol given application protocol can be found */
   ndpi_l4_proto_info ndpi_get_l4_proto_info(struct ndpi_detection_module_struct *ndpi_struct, u_int16_t ndpi_proto_id);
   const char* ndpi_get_l4_proto_name(ndpi_l4_proto_info proto);
-    
+
   ndpi_proto_defaults_t* ndpi_get_proto_defaults(struct ndpi_detection_module_struct *ndpi_mod);
   u_int ndpi_get_ndpi_num_supported_protocols(struct ndpi_detection_module_struct *ndpi_mod);
   u_int ndpi_get_ndpi_num_custom_protocols(struct ndpi_detection_module_struct *ndpi_mod);
   u_int ndpi_get_ndpi_detection_module_size(void);
   void ndpi_set_log_level(struct ndpi_detection_module_struct *ndpi_mod, u_int l);
+  void ndpi_set_debug_bitmask(struct ndpi_detection_module_struct *ndpi_mod, NDPI_PROTOCOL_BITMASK debug_bitmask);
 
   /* LRU cache */
   struct ndpi_lru_cache* ndpi_lru_cache_init(u_int32_t num_entries);
@@ -810,18 +858,21 @@ extern "C" {
   u_int8_t ndpi_lru_find_cache(struct ndpi_lru_cache *c, u_int32_t key,
 			       u_int16_t *value, u_int8_t clean_key_when_found);
   void ndpi_lru_add_to_cache(struct ndpi_lru_cache *c, u_int32_t key, u_int16_t value);
-  
+
   /**
-   * Add a string to match to an automata
+   * Find a protocol id associated with a string automata
    *
    * @par     The automata initialized with ndpi_init_automa();
    * @par     The (sub)string to search
    * @par     The (sub)string length
-   * @par     The id associated with the matched string or 0 id not found.
+   * @par     The protocol id associated with the matched string or 0 id not found.
    * @return  0 in case of match, or -1 if no match, or -2 if an error occurred.
    *
    */
-  int ndpi_match_string_id(void *_automa, char *string_to_match, u_int match_len, unsigned long *id);
+  int ndpi_match_string_protocol_id(void *_automa, char *string_to_match, u_int match_len,
+				    u_int16_t *protocol_id,
+				    ndpi_protocol_category_t *category,
+				    ndpi_protocol_breed_t *breed);
 
   /* Utility functions to set ndpi malloc/free/print wrappers */
   void set_ndpi_malloc(void* (*__ndpi_malloc)(size_t size));
@@ -833,7 +884,8 @@ extern "C" {
   //void * ndpi_malloc(size_t size);
   //void * ndpi_calloc(unsigned long count, size_t size);
   //void ndpi_free(void *ptr);
-  u_int8_t ndpi_get_api_version(void);
+  u_int16_t ndpi_get_api_version(void);
+  const char *ndpi_get_gcrypt_version(void);
 
   /* https://github.com/corelight/community-id-spec */
   int ndpi_flowv4_flow_hash(u_int8_t l4_proto, u_int32_t src_ip, u_int32_t dst_ip, u_int16_t src_port, u_int16_t dst_port,
@@ -842,7 +894,7 @@ extern "C" {
 			    u_int16_t src_port, u_int16_t dst_port, u_int8_t icmp_type, u_int8_t icmp_code,
 			    u_char *hash_buf, u_int8_t hash_buf_len);
   u_int8_t ndpi_extra_dissection_possible(struct ndpi_detection_module_struct *ndpi_struct,
-					  struct ndpi_flow_struct *flow);    
+					  struct ndpi_flow_struct *flow);
   u_int8_t ndpi_is_safe_ssl_cipher(u_int32_t cipher);
   const char* ndpi_cipher2str(u_int32_t cipher);
   const char* ndpi_tunnel2str(ndpi_packet_tunnel tt);
@@ -852,15 +904,21 @@ extern "C" {
 				      char *buffer, u_int buffer_size,
 				      u_int8_t min_string_match_len, /* Will return 0 if no string > min_string_match_len have been found */
 				      char *outbuf, u_int outbuf_len);
-  char* ndpi_ssl_version2str(u_int16_t version, u_int8_t *unknown_tls_version);
+  char* ndpi_ssl_version2str(struct ndpi_flow_struct *flow,
+                             u_int16_t version, u_int8_t *unknown_tls_version);
   void ndpi_patchIPv6Address(char *str);
   void ndpi_user_pwd_payload_copy(u_int8_t *dest, u_int dest_len, u_int offset,
 				  const u_int8_t *src, u_int src_len);
   u_char* ndpi_base64_decode(const u_char *src, size_t len, size_t *out_len);
-  char* ndpi_base64_encode(unsigned char const* bytes_to_encode, ssize_t in_len);
+  char* ndpi_base64_encode(unsigned char const* bytes_to_encode, size_t in_len); /* NOTE: caller MUST free the returned pointer */
+  void ndpi_string_sha1_hash(const uint8_t *message, size_t len, u_char *hash /* 20-bytes */);
+
   int ndpi_load_ipv4_ptree(struct ndpi_detection_module_struct *ndpi_str,
 			   const char *path, u_int16_t protocol_id);
-    
+  int ndpi_dpi2json(struct ndpi_detection_module_struct *ndpi_struct,
+		    struct ndpi_flow_struct *flow,
+		    ndpi_protocol l7_protocol,
+		    ndpi_serializer *serializer);
   int ndpi_flow2json(struct ndpi_detection_module_struct *ndpi_struct,
 		     struct ndpi_flow_struct *flow,
 		     u_int8_t ip_version,
@@ -873,80 +931,417 @@ extern "C" {
 
   void ndpi_md5(const u_char *data, size_t data_len, u_char hash[16]);
 
+  const char* ndpi_http_method2str(ndpi_http_method m);
+  ndpi_http_method ndpi_http_str2method(const char* method, ssize_t method_len);
+  
   /* ptree (trie) API */
   ndpi_ptree_t* ndpi_ptree_create(void);
-  int ndpi_ptree_insert(ndpi_ptree_t *tree, const ndpi_ip_addr_t *addr, u_int8_t bits, uint user_data);
-  int ndpi_ptree_match_addr(ndpi_ptree_t *tree, const ndpi_ip_addr_t *addr, uint *user_data);
+  int ndpi_ptree_insert(ndpi_ptree_t *tree, const ndpi_ip_addr_t *addr, u_int8_t bits, u_int32_t user_data);
+  int ndpi_ptree_match_addr(ndpi_ptree_t *tree, const ndpi_ip_addr_t *addr, u_int32_t *user_data);
   void ndpi_ptree_destroy(ndpi_ptree_t *tree);
 
-  /* Serializer */
-  int ndpi_init_serializer_ll(ndpi_serializer *serializer, ndpi_serialization_format fmt,
-			      u_int32_t buffer_size);
+  /* General purpose utilities */
+  u_int64_t ndpi_htonll(u_int64_t v);
+  u_int64_t ndpi_ntohll(u_int64_t v);
+
+  /* DGA */
+  int ndpi_check_dga_name(struct ndpi_detection_module_struct *ndpi_str,
+			  struct ndpi_flow_struct *flow,
+			  char *name, u_int8_t is_hostname);
+
+  /* Serializer (supports JSON, TLV, CSV) */
+
+  /**
+   * Initialize a serializer handle (allocated by the caller).
+   * @param serializer The serializer handle 
+   * @param fmt The serialization format (ndpi_serialization_format_json, ndpi_serialization_format_tlv, ndpi_serialization_format_csv)
+   * @return 0 on success, a negative number otherwise
+   */
   int ndpi_init_serializer(ndpi_serializer *serializer, ndpi_serialization_format fmt);
+
+  /**
+   * Initialize a serializer handle. Same as ndpi_init_serializer, but with some low-level settings.
+   * @param serializer The serializer handle 
+   * @param fmt The serialization format (ndpi_serialization_format_json, ndpi_serialization_format_tlv, ndpi_serialization_format_csv)
+   * @param buffer_size The initial internal buffer_size
+   * @return 0 on success, a negative number otherwise 
+   */
+  int ndpi_init_serializer_ll(ndpi_serializer *serializer, ndpi_serialization_format fmt, u_int32_t buffer_size);
+
+  /**
+   * Release all allocated data structure.
+   * @param serializer The serializer handle 
+   */
   void ndpi_term_serializer(ndpi_serializer *serializer);
+
+  /**
+   * Reset the serializer (cleanup the internal buffer to start a new serialization)
+   * @param serializer The serializer handle 
+   */
   void ndpi_reset_serializer(ndpi_serializer *serializer);
 
-  int ndpi_serialize_uint32_uint32(ndpi_serializer *serializer,
-				   u_int32_t key, u_int32_t value);
-  int ndpi_serialize_uint32_uint64(ndpi_serializer *serializer,
-				   u_int32_t key, u_int64_t value);
-  int ndpi_serialize_uint32_int32(ndpi_serializer *serializer,
-				  u_int32_t key, int32_t value);
-  int ndpi_serialize_uint32_int64(ndpi_serializer *serializer,
-				  u_int32_t key, int64_t value);
-  int ndpi_serialize_uint32_float(ndpi_serializer *serializer,
-				  u_int32_t key, float value,
-				  const char *format /* e.f. "%.2f" */);
-  int ndpi_serialize_uint32_string(ndpi_serializer *serializer,
-				   u_int32_t key, const char *value);
-  int ndpi_serialize_uint32_boolean(ndpi_serializer *serializer,
-				    u_int32_t key, u_int8_t value);
+  /**
+   * Serialize a 32-bit unsigned int key and a 32-bit unsigned int value
+   * @param serializer The serializer handle 
+   * @param key The field name or ID
+   * @param value The field value
+   * @return 0 on success, a negative number otherwise
+   */
+  int ndpi_serialize_uint32_uint32(ndpi_serializer *serializer, u_int32_t key, u_int32_t value);
 
-  int ndpi_serialize_string_int32(ndpi_serializer *serializer,
-				  const char *key, int32_t value);
-  int ndpi_serialize_string_int64(ndpi_serializer *serializer,
-				  const char *key, int64_t value);  
-  int ndpi_serialize_string_uint32(ndpi_serializer *serializer,
-				   const char *key, u_int32_t value);
-  int ndpi_serialize_string_uint32_format(ndpi_serializer *serializer,
-					  const char *key, u_int32_t value,
-					  const char *format);
-  int ndpi_serialize_string_uint64(ndpi_serializer *serializer,
-				   const char *key, u_int64_t value);
-  int ndpi_serialize_string_string(ndpi_serializer *serializer,
-				   const char *key, const char *value);
-  int ndpi_serialize_string_binary(ndpi_serializer *serializer,
-				   const char *key, const char *_value,
-				   u_int16_t vlen);
-  int ndpi_serialize_string_raw(ndpi_serializer *_serializer,
-				const char *key, const char *_value,
-                                u_int16_t vlen);
-  int ndpi_serialize_string_float(ndpi_serializer *serializer,
-				  const char *key, float value,
-				  const char *format /* e.f. "%.2f" */);
-  int ndpi_serialize_string_boolean(ndpi_serializer *serializer,
-				    const char *key, u_int8_t value);
+  /**
+   * Serialize a 32-bit unsigned int key and a 64-bit unsigned int value 
+   * @param serializer The serializer handle 
+   * @param key The field name or ID
+   * @param value The field value
+   * @return 0 on success, a negative number otherwise
+   */
+  int ndpi_serialize_uint32_uint64(ndpi_serializer *serializer, u_int32_t key, u_int64_t value);
 
+  /**
+   * Serialize a 32-bit unsigned int key and a 32-bit signed int value
+   * @param serializer The serializer handle 
+   * @param key The field name or ID
+   * @param value The field value
+   * @return 0 on success, a negative number otherwise
+   */
+  int ndpi_serialize_uint32_int32(ndpi_serializer *serializer, u_int32_t key, int32_t value);
+
+  /**
+   * Serialize a 32-bit unsigned int key and a 64-bit signed int value
+   * @param serializer The serializer handle 
+   * @param key The field name or ID
+   * @param value The field value
+   * @return 0 on success, a negative number otherwise
+   */
+  int ndpi_serialize_uint32_int64(ndpi_serializer *serializer, u_int32_t key, int64_t value);
+
+  /**
+   * Serialize a 32-bit unsigned int key and a float value
+   * @param serializer The serializer handle 
+   * @param key The field name or ID
+   * @param value The field value
+   * @param format The float value format
+   * @return 0 on success, a negative number otherwise
+   */
+  int ndpi_serialize_uint32_float(ndpi_serializer *serializer, u_int32_t key, float value, const char *format /* e.f. "%.2f" */);
+
+  /**
+   * Serialize a 32-bit unsigned int key and a string value
+   * @param serializer The serializer handle 
+   * @param key The field name or ID
+   * @param value The field value
+   * @return 0 on success, a negative number otherwise
+   */
+  int ndpi_serialize_uint32_string(ndpi_serializer *serializer, u_int32_t key, const char *value);
+
+  /**
+   * Serialize a 32-bit unsigned int key and a boolean value
+   * @param serializer The serializer handle 
+   * @param key The field name or ID
+   * @param value The field value
+   * @return 0 on success, a negative number otherwise
+   */
+  int ndpi_serialize_uint32_boolean(ndpi_serializer *serializer, u_int32_t key, u_int8_t value);
+
+  /**
+   * Serialize an unterminated string key and a 32-bit signed int value
+   * @param serializer The serializer handle 
+   * @param key The field name or ID
+   * @param klen The key length
+   * @param value The field value
+   * @return 0 on success, a negative number otherwise
+   */
+  int ndpi_serialize_binary_int32(ndpi_serializer *_serializer, const char *key, u_int16_t klen, int32_t value);
+
+  /**
+   * Serialize a string key and a 32-bit signed int value
+   * @param serializer The serializer handle 
+   * @param key The field name or ID
+   * @param value The field value
+   * @return 0 on success, a negative number otherwise
+   */
+  int ndpi_serialize_string_int32(ndpi_serializer *serializer, const char *key, int32_t value);
+
+  /**
+   * Serialize an unterminated string key and a 64-bit signed int value
+   * @param serializer The serializer handle 
+   * @param key The field name or ID
+   * @param klen The key length
+   * @param value The field value
+   * @return 0 on success, a negative number otherwise
+   */
+  int ndpi_serialize_binary_int64(ndpi_serializer *_serializer, const char *key, u_int16_t klen, int64_t value);
+
+  /**
+   * Serialize a string key and a 64-bit signed int value
+   * @param serializer The serializer handle 
+   * @param key The field name or ID
+   * @param value The field value
+   * @return 0 on success, a negative number otherwise
+   */
+  int ndpi_serialize_string_int64(ndpi_serializer *serializer, const char *key, int64_t value);
+
+  /**
+   * Serialize an unterminated string key and a 32-bit unsigned int value
+   * @param serializer The serializer handle 
+   * @param key The field name or ID
+   * @param klen The key length
+   * @param value The field value
+   * @return 0 on success, a negative number otherwise
+   */
+  int ndpi_serialize_binary_uint32(ndpi_serializer *_serializer, const char *key, u_int16_t klen, u_int32_t value);
+
+  /**
+   * Serialize a string key and a 32-bit unsigned int value
+   * @param serializer The serializer handle 
+   * @param key The field name or ID
+   * @param value The field value
+   * @return 0 on success, a negative number otherwise
+   */
+  int ndpi_serialize_string_uint32(ndpi_serializer *serializer, const char *key, u_int32_t value);
+
+  /**
+   * Serialize a string key and a float value
+   * @param serializer The serializer handle 
+   * @param key The field name or ID
+   * @param value The field value
+   * @param format The float format
+   * @return 0 on success, a negative number otherwise
+   */
+  int ndpi_serialize_string_uint32_format(ndpi_serializer *serializer, const char *key, u_int32_t value, const char *format);
+
+  /**
+   * Serialize an unterminated string key and a 64-bit unsigned int value
+   * @param serializer The serializer handle 
+   * @param key The field name or ID
+   * @param klen The key length
+   * @param value The field value
+   * @return 0 on success, a negative number otherwise
+   */
+  int ndpi_serialize_binary_uint64(ndpi_serializer *_serializer, const char *key, u_int16_t klen, u_int64_t value);
+
+  /**
+   * Serialize a string key and a 64-bit unsigned int value
+   * @param serializer The serializer handle 
+   * @param key The field name or ID
+   * @param value The field value
+   * @return 0 on success, a negative number otherwise
+   */
+  int ndpi_serialize_string_uint64(ndpi_serializer *serializer, const char *key, u_int64_t value);
+
+  /**
+   * Serialize an unterminated string key and an unterminated string value
+   * @param serializer The serializer handle 
+   * @param key The field name or ID
+   * @param klen The key length
+   * @param value The field value
+   * @param vlen The value length
+   * @return 0 on success, a negative number otherwise
+   */
+  int ndpi_serialize_binary_binary(ndpi_serializer *_serializer, const char *key, u_int16_t klen, const char *_value, u_int16_t vlen);
+
+  /**
+   * Serialize a string key and a string value
+   * @param serializer The serializer handle 
+   * @param key The field name or ID
+   * @param value The field value
+   * @return 0 on success, a negative number otherwise
+   */
+  int ndpi_serialize_string_string(ndpi_serializer *serializer, const char *key, const char *value);
+
+  /**
+   * Serialize a string key and an unterminated string value
+   * @param serializer The serializer handle 
+   * @param key The field name or ID
+   * @param value The field value
+   * @param vlen The value length
+   * @return 0 on success, a negative number otherwise
+   */
+  int ndpi_serialize_string_binary(ndpi_serializer *serializer, const char *key, const char *_value, u_int16_t vlen);
+
+  /**
+   * Serialize a string key and a raw value (this is a string which is added to the JSON without any quote or escaping)
+   * @param serializer The serializer handle 
+   * @param key The field name or ID
+   * @param value The field value
+   * @param vlen The value length
+   * @return 0 on success, a negative number otherwise
+   */
+  int ndpi_serialize_string_raw(ndpi_serializer *_serializer, const char *key, const char *_value, u_int16_t vlen);
+
+  /**
+   * Serialize an unterminated string key and a float value
+   * @param serializer The serializer handle 
+   * @param key The field name or ID
+   * @param klen The key length
+   * @param value The field value
+   * @param format The float format
+   * @return 0 on success, a negative number otherwise
+   */
+  int ndpi_serialize_binary_float(ndpi_serializer *_serializer, const char *key, u_int16_t klen, float value, const char *format /* e.f. "%.2f" */);
+
+  /**
+   * Serialize a string key and a a float value 
+   * @param serializer The serializer handle 
+   * @param key The field name or ID
+   * @param value The field value
+   * @param format The float format
+   * @return 0 on success, a negative number otherwise
+   */
+  int ndpi_serialize_string_float(ndpi_serializer *serializer, const char *key, float value, const char *format /* e.f. "%.2f" */);
+
+  /**
+   * Serialize a string key and a boolean value
+   * @param serializer The serializer handle 
+   * @param key The field name or ID
+   * @param value The field value
+   * @return 0 on success, a negative number otherwise
+   */
+  int ndpi_serialize_string_boolean(ndpi_serializer *serializer, const char *key, u_int8_t value);
+
+  /**
+   * Serialize a raw record in an array (this is a low-level function and its use is not recommended)
+   * @param serializer The serializer handle 
+   * @param record The record value
+   * @param record_len The record length
+   * @return 0 on success, a negative number otherwise
+   */
+  int ndpi_serialize_raw_record(ndpi_serializer *_serializer, u_char *record, u_int32_t record_len);
+
+  /**
+   * Serialize an End-Of-Record (the current object becomes is terminated and added to an array,
+   * and a new object is created where the next items will be added)
+   * @param serializer The serializer handle 
+   * @return 0 on success, a negative number otherwise
+   */
   int ndpi_serialize_end_of_record(ndpi_serializer *serializer);
-  int ndpi_serialize_start_of_block(ndpi_serializer *serializer,
-				    const char *key);
+
+  /**
+   * Serialize the start of a list with an unterminated string key, where the next serialized items 
+   * will be added (note: keys for the new items are ignored)
+   * @param serializer The serializer handle 
+   * @param key The field name or ID
+   * @param klen The key length
+   * @return 0 on success, a negative number otherwise
+   */
+  int ndpi_serialize_start_of_list_binary(ndpi_serializer *_serializer, const char *key, u_int16_t klen);
+
+  /**
+   * Serialize the start of a list, where the next serialized items will be added (note: keys for
+   * the new items are ignored)
+   * @param serializer The serializer handle 
+   * @param key The field name or ID
+   * @param value The field value
+   * @return 0 on success, a negative number otherwise
+   */
+  int ndpi_serialize_start_of_list(ndpi_serializer *serializer, const char *key);
+
+  /**
+   * Serialize the end of a list
+   * @param serializer The serializer handle 
+   * @return 0 on success, a negative number otherwise
+   */
+  int ndpi_serialize_end_of_list(ndpi_serializer *serializer);
+
+  /**
+   * Serialize the start of a block with an unterminated string key 
+   * @param serializer The serializer handle 
+   * @param key The field name or ID
+   * @param klen The key length
+   * @return 0 on success, a negative number otherwise
+   */
+  int ndpi_serialize_start_of_block_binary(ndpi_serializer *_serializer, const char *key, u_int16_t klen);
+
+  /**
+   * Serialize the start of a block with a string key 
+   * @param serializer The serializer handle 
+   * @param key The field name or ID
+   * @return 0 on success, a negative number otherwise
+   */
+  int ndpi_serialize_start_of_block(ndpi_serializer *serializer, const char *key);
+
+  /**
+   * Serialize the end of a block 
+   * @param serializer The serializer handle 
+   * @param key The field name or ID
+   * @param value The field value
+   * @return 0 on success, a negative number otherwise
+   */
   int ndpi_serialize_end_of_block(ndpi_serializer *serializer);
+
+  /**
+   * Return the serialized buffer
+   * @param serializer The serializer handle 
+   * @param buffer_len The buffer length (out)
+   * @return The buffer
+   */
   char* ndpi_serializer_get_buffer(ndpi_serializer *serializer, u_int32_t *buffer_len);
+
+  /**
+   * Return the current serialized buffer length
+   * @param serializer The serializer handle 
+   * @return The buffer length
+   */
   u_int32_t ndpi_serializer_get_buffer_len(ndpi_serializer *serializer);
+
+  /**
+   * Return the real internal buffer size (containing the serialized buffer)
+   * @param serializer The serializer handle
+   * @return The internal buffer size
+   */
   u_int32_t ndpi_serializer_get_internal_buffer_size(ndpi_serializer *serializer);
+
+  /**
+   * Change the serializer buffer length
+   * @param serializer The serializer handle 
+   * @param l The new buffer length 
+   * @return 0 on success, a negative number otherwise
+   */
   int ndpi_serializer_set_buffer_len(ndpi_serializer *serializer, u_int32_t l);
+
+  /**
+   * Return the configured serialization format
+   * @param serializer The serializer handle 
+   * @return The serialization format
+   */
+  ndpi_serialization_format ndpi_serializer_get_format(ndpi_serializer *serializer);
+
+  /**
+   * Set the CSV separator
+   * @param serializer The serializer handle 
+   * @param separator The separator
+   */
   void ndpi_serializer_set_csv_separator(ndpi_serializer *serializer, char separator);
 
+  /**
+   * Return the header automatically built from keys (CSV only)
+   * @param serializer The serializer handle 
+   * @param buffer_len The buffer length (out)
+   * @return The header
+   */
+  char* ndpi_serializer_get_header(ndpi_serializer *serializer, u_int32_t *buffer_len);
+
+  /**
+   * Create a snapshot of the internal buffer for later rollback (ndpi_serializer_rollback_snapshot)
+   * @param serializer The serializer handle 
+   */
   void ndpi_serializer_create_snapshot(ndpi_serializer *serializer);
+
+  /**
+   * Rollback to the latest snapshot
+   * @param serializer The serializer handle 
+   */
   void ndpi_serializer_rollback_snapshot(ndpi_serializer *serializer);
-  
-  /* Deserializer */
+
+  /* Deserializer (supports TLV only) */
+
   int ndpi_init_deserializer(ndpi_deserializer *deserializer,
 			     ndpi_serializer *serializer);
   int ndpi_init_deserializer_buf(ndpi_deserializer *deserializer,
 				 u_int8_t *serialized_buffer,
 				 u_int32_t serialized_buffer_len);
-  
+
   ndpi_serialization_format ndpi_deserialize_get_format(ndpi_deserializer *_deserializer);
   ndpi_serialization_type ndpi_deserialize_get_item_type(ndpi_deserializer *deserializer, ndpi_serialization_type *key_type);
   int ndpi_deserialize_next(ndpi_deserializer *deserializer);
@@ -968,26 +1363,69 @@ extern "C" {
   struct ndpi_analyze_struct* ndpi_alloc_data_analysis(u_int16_t _max_series_len);
   void ndpi_init_data_analysis(struct ndpi_analyze_struct *s, u_int16_t _max_series_len);
   void ndpi_free_data_analysis(struct ndpi_analyze_struct *d);
+  void ndpi_reset_data_analysis(struct ndpi_analyze_struct *d);
   void ndpi_data_add_value(struct ndpi_analyze_struct *s, const u_int32_t value);
 
-  float ndpi_data_average(struct ndpi_analyze_struct *s);
+  /* Sliding-window only */
   float ndpi_data_window_average(struct ndpi_analyze_struct *s);
-  
+  float ndpi_data_window_variance(struct ndpi_analyze_struct *s);
+  float ndpi_data_window_stddev(struct ndpi_analyze_struct *s);
+
+  /* All data */
+  float ndpi_data_average(struct ndpi_analyze_struct *s);
   float ndpi_data_entropy(struct ndpi_analyze_struct *s);
   float ndpi_data_variance(struct ndpi_analyze_struct *s);
   float ndpi_data_stddev(struct ndpi_analyze_struct *s);
+  u_int32_t ndpi_data_last(struct ndpi_analyze_struct *s);
   u_int32_t ndpi_data_min(struct ndpi_analyze_struct *s);
   u_int32_t ndpi_data_max(struct ndpi_analyze_struct *s);
   float ndpi_data_ratio(u_int32_t sent, u_int32_t rcvd);
-    
+
   const char* ndpi_data_ratio2str(float ratio);
-  
+
   void ndpi_data_print_window_values(struct ndpi_analyze_struct *s); /* debug */
 
-  ndpi_url_risk ndpi_validate_url(char *url);
+  ndpi_risk_enum ndpi_validate_url(char *url);
 
   u_int8_t ndpi_is_protocol_detected(struct ndpi_detection_module_struct *ndpi_str,
 				     ndpi_protocol proto);
+  void ndpi_serialize_risk(ndpi_serializer *serializer, struct ndpi_flow_struct *flow);
+
+  const char* ndpi_risk2str(ndpi_risk_enum risk);
+
+  /* ******************************* */
+
+  /* HyperLogLog cardinality estimator */
+
+  /* Memory lifecycle */
+  int ndpi_hll_init(struct ndpi_hll *hll, u_int8_t bits);
+  void ndpi_hll_destroy(struct ndpi_hll *hll);
+  void ndpi_hll_reset(struct ndpi_hll *hll);
+  
+  /* Add values */
+  void ndpi_hll_add(struct ndpi_hll *hll, const char *data, size_t data_len);
+  void ndpi_hll_add_number(struct ndpi_hll *hll, u_int32_t value) ;
+
+  /* Get cardinality estimation */
+  double ndpi_hll_count(struct ndpi_hll *hll);
+
+  /* ******************************* */
+
+  int  ndpi_init_bin(struct ndpi_bin *b, enum ndpi_bin_family f, u_int8_t num_bins);
+  void ndpi_free_bin(struct ndpi_bin *b);
+  struct ndpi_bin* ndpi_clone_bin(struct ndpi_bin *b);
+  void ndpi_inc_bin(struct ndpi_bin *b, u_int8_t slot_id, u_int32_t val);
+  void ndpi_set_bin(struct ndpi_bin *b, u_int8_t slot_id, u_int32_t value);
+  u_int32_t ndpi_get_bin_value(struct ndpi_bin *b, u_int8_t slot_id);
+  void ndpi_reset_bin(struct ndpi_bin *b);
+  void ndpi_normalize_bin(struct ndpi_bin *b);
+  char* ndpi_print_bin(struct ndpi_bin *b, u_int8_t normalize_first, char *out_buf, u_int out_buf_len);
+  float ndpi_bin_similarity(struct ndpi_bin *b1, struct ndpi_bin *b2, u_int8_t normalize_first);
+  int ndpi_cluster_bins(struct ndpi_bin *bins, u_int16_t num_bins,
+			u_int8_t num_clusters, u_int16_t *cluster_ids,
+			struct ndpi_bin *centroids);
+
+  u_int32_t ndpi_quick_16_byte_hash(u_int8_t *in_16_bytes_long);
 #ifdef __cplusplus
 }
 #endif
