@@ -1,59 +1,46 @@
 <?php
 
-/**
- *    Copyright (C) 2015 Deciso B.V.
+/*
+ * Copyright (C) 2015-2019 Deciso B.V.
+ * All rights reserved.
  *
- *    All rights reserved.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
  *
- *    Redistribution and use in source and binary forms, with or without
- *    modification, are permitted provided that the following conditions are met:
+ * 1. Redistributions of source code must retain the above copyright notice,
+ *    this list of conditions and the following disclaimer.
  *
- *    1. Redistributions of source code must retain the above copyright notice,
- *       this list of conditions and the following disclaimer.
- *
- *    2. Redistributions in binary form must reproduce the above copyright
- *       notice, this list of conditions and the following disclaimer in the
- *       documentation and/or other materials provided with the distribution.
- *
- *    THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
- *    INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
- *    AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
- *    AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
- *    OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- *    SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- *    INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- *    CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- *    ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- *    POSSIBILITY OF SUCH DAMAGE.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
  *
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
+ * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
+ * AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
+ * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
  */
+
 namespace OPNsense\Base\FieldTypes;
 
-use Phalcon\Validation\Validator\InclusionIn;
 use OPNsense\Core\Config;
-use OPNsense\Base\Validators\CsvListValidator;
 
 /**
  * Class InterfaceField field type to select usable interfaces, currently this is kind of a backward compatibility
  * package to glue legacy interfaces into the model.
  * @package OPNsense\Base\FieldTypes
  */
-class InterfaceField extends BaseField
+class InterfaceField extends BaseListField
 {
-    /**
-     * @var bool marks if this is a data node or a container
-     */
-    protected $internalIsContainer = false;
-
-    /**
-     * @var string default validation message string
-     */
-    protected $internalValidationMessage = "please specify a valid interface";
-
     /**
      * @var array collected options
      */
-    private static $internalOptionList = array();
+    private static $internalStaticOptionList = array();
 
     /**
      * @var array filters to use on the interface list
@@ -65,11 +52,6 @@ class InterfaceField extends BaseField
      */
     private $internalCacheKey = '*';
 
-    /**
-     * @var bool field may contain multiple interfaces at once
-     */
-    private $internalMultiSelect = false;
-
     /**
      * @var bool add physical interfaces to selection (collected from lagg, vlan)
      */
@@ -78,7 +60,7 @@ class InterfaceField extends BaseField
     /**
      * @var bool allow dynamic interfaces
      */
-     private $internalAllowDynamic = false;
+    private $internalAllowDynamic = 0;
 
     /**
      *  collect parents for lagg interfaces
@@ -127,8 +109,8 @@ class InterfaceField extends BaseField
      */
     protected function actionPostLoadingEvent()
     {
-        if (!isset(self::$internalOptionList[$this->internalCacheKey])) {
-            self::$internalOptionList[$this->internalCacheKey] = array();
+        if (!isset(self::$internalStaticOptionList[$this->internalCacheKey])) {
+            self::$internalStaticOptionList[$this->internalCacheKey] = array();
 
             $allInterfaces = array();
             $allInterfacesDevices = array(); // mapping device -> interface handle (lan/wan/optX)
@@ -138,6 +120,10 @@ class InterfaceField extends BaseField
                 foreach ($configObj->interfaces->children() as $key => $value) {
                     if (!$this->internalAllowDynamic && !empty($value->internal_dynamic)) {
                         continue;
+                    } elseif ($this->internalAllowDynamic == 2 && !empty($value->internal_dynamic)) {
+                        if (empty($value->ipaddr) && empty($value->ipaddrv6)) {
+                            continue;
+                        }
                     }
                     $allInterfaces[$key] = $value;
                     if (!empty($value->if)) {
@@ -191,14 +177,22 @@ class InterfaceField extends BaseField
                     }
                 }
                 if ($isMatched) {
-                    self::$internalOptionList[$this->internalCacheKey][$key] =
+                    self::$internalStaticOptionList[$this->internalCacheKey][$key] =
                         !empty($value->descr) ? (string)$value->descr : strtoupper($key);
                 }
             }
-            natcasesort(self::$internalOptionList[$this->internalCacheKey]);
+            natcasesort(self::$internalStaticOptionList[$this->internalCacheKey]);
         }
+        $this->internalOptionList = self::$internalStaticOptionList[$this->internalCacheKey];
     }
 
+    private function updateInternalCacheKey()
+    {
+        $tmp  = serialize($this->internalFilters);
+        $tmp .= (string)$this->internalAllowDynamic;
+        $tmp .= $this->internalAddParentDevices ? "Y" : "N";
+        $this->internalCacheKey = md5($tmp);
+    }
     /**
      * set filters to use (in regex) per field, all tags are combined
      * and cached for the next object using the same filters
@@ -208,7 +202,7 @@ class InterfaceField extends BaseField
     {
         if (is_array($filters)) {
             $this->internalFilters = $filters;
-            $this->internalCacheKey = md5(serialize($this->internalFilters));
+            $this->updateInternalCacheKey();
         }
     }
 
@@ -223,78 +217,22 @@ class InterfaceField extends BaseField
         } else {
             $this->internalAddParentDevices = false;
         }
-    }
-
-    /**
-     * select if multiple interfaces may be selected at once
-     * @param $value boolean value 0/1
-     */
-    public function setMultiple($value)
-    {
-        if (trim(strtoupper($value)) == "Y") {
-            $this->internalMultiSelect = true;
-        } else {
-            $this->internalMultiSelect = false;
-        }
+        $this->updateInternalCacheKey();
     }
 
     /**
      * select if dynamic (hotplug) interfaces maybe selectable
-     * @param $value boolean value 0/1
+     * @param $value Y/N/S (Yes, No, Static)
      */
     public function setAllowDynamic($value)
     {
         if (trim(strtoupper($value)) == "Y") {
-            $this->internalAllowDynamic = true;
+            $this->internalAllowDynamic = 1;
+        } elseif (trim(strtoupper($value)) == "S") {
+            $this->internalAllowDynamic = 2;
         } else {
-            $this->internalAllowDynamic = false;
+            $this->internalAllowDynamic = 0;
         }
-    }
-
-    /**
-     * get valid options, descriptions and selected value
-     * @return array
-     */
-    public function getNodeData()
-    {
-        $result = array();
-        // if interface is not required and single, add empty option
-        if (!$this->internalIsRequired && !$this->internalMultiSelect) {
-            $result[""] = array("value" => gettext("none"), "selected" => 0);
-        }
-
-        // explode interfaces
-        $interfaces = explode(',', $this->internalValue);
-        foreach (self::$internalOptionList[$this->internalCacheKey] as $optKey => $optValue) {
-            if (in_array($optKey, $interfaces)) {
-                $selected = 1;
-            } else {
-                $selected = 0;
-            }
-            $result[$optKey] = array("value" => $optValue, "selected" => $selected);
-        }
-
-        return $result;
-    }
-
-    /**
-     * retrieve field validators for this field type
-     * @return array returns validators
-     */
-    public function getValidators()
-    {
-        $validators = parent::getValidators();
-        if ($this->internalValue != null) {
-            if ($this->internalMultiSelect) {
-                // field may contain more than one interface
-                $validators[] = new CsvListValidator(array('message' => $this->internalValidationMessage,
-                    'domain'=>array_keys(self::$internalOptionList[$this->internalCacheKey])));
-            } else {
-                // single interface selection
-                $validators[] = new InclusionIn(array('message' => $this->internalValidationMessage,
-                    'domain'=>array_keys(self::$internalOptionList[$this->internalCacheKey])));
-            }
-        }
-        return $validators;
+        $this->updateInternalCacheKey();
     }
 }
