@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 """
-Copyright (c) 2014-2020 Maltrail developers (https://github.com/stamparm/maltrail/)
+Copyright (c) 2014-2021 Maltrail developers (https://github.com/stamparm/maltrail/)
 See the file 'LICENSE' for copying permission
 """
 from __future__ import print_function
@@ -96,12 +96,12 @@ def update_trails(force=False, offline=False):
 
     else:
         trail_files = set()
-        for dirpath, dirnames, filenames in os.walk(os.path.abspath(os.path.join(ROOT_DIR, "trails"))) :
+        for dirpath, dirnames, filenames in os.walk(os.path.abspath(os.path.join(ROOT_DIR, "trails"))):
             for filename in filenames:
                 trail_files.add(os.path.abspath(os.path.join(dirpath, filename)))
 
         if config.CUSTOM_TRAILS_DIR:
-            for dirpath, dirnames, filenames in os.walk(os.path.abspath(os.path.join(ROOT_DIR, os.path.expanduser(config.CUSTOM_TRAILS_DIR)))) :
+            for dirpath, dirnames, filenames in os.walk(os.path.abspath(os.path.join(ROOT_DIR, os.path.expanduser(config.CUSTOM_TRAILS_DIR)))):
                 for filename in filenames:
                     trail_files.add(os.path.abspath(os.path.join(dirpath, filename)))
 
@@ -182,7 +182,7 @@ def update_trails(force=False, offline=False):
                     if not url:
                         continue
 
-                    url = ("http://%s" % url) if not "//" in url else url
+                    url = ("http://%s" % url) if "//" not in url else url
                     content = retrieve_content(url)
 
                     if not content:
@@ -263,6 +263,10 @@ def update_trails(force=False, offline=False):
                             del trails[key]
                             continue
 
+                    if any(int(_) > 255 for _ in key.split('.')):
+                        del trails[key]
+                        continue
+
                 if trails[key][0] == "ransomware":
                     trails[key] = ("ransomware (malware)", trails[key][1])
 
