@@ -1,17 +1,18 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 
 """
-Copyright (c) 2014-2019 Maltrail developers (https://github.com/stamparm/maltrail/)
+Copyright (c) 2014-2021 Maltrail developers (https://github.com/stamparm/maltrail/)
 See the file 'LICENSE' for copying permission
 """
 
 from core.common import retrieve_content
 
-__url__ = "https://feodotracker.abuse.ch/blocklist/?download=ipblocklist"
+__url__ = "https://feodotracker.abuse.ch/downloads/ipblocklist_recommended.txt"
 __check__ = "Feodo"
-__info__ = "feodo (malware)"
+__info__ = "emotet (malware)"
 __reference__ = "abuse.ch"
 
+# Note: "Feodo malware family (Dridex, Emotet/Heodo)" <- actually, only tracking Emotet variant"
 def fetch():
     retval = {}
     content = retrieve_content(__url__)
