@@ -24,24 +24,6 @@ if ntop.isPro() then
   end
 end
 
-local function getAlertEndpointsEntries()
-  local endpoints = plugins_utils.getLoadedAlertEndpoints()
-  local rv = {
-    notification_severity_preference = {
-      title       = i18n("prefs.slack_notification_severity_preference_title", {url="http://www.slack.com"}),
-      description = i18n("prefs.slack_notification_severity_preference_description"),
-    }
-  }
-
-  for _, endpoint in ipairs(endpoints) do
-    for entry_k, entry in pairs(endpoint.prefs_entries or {}) do
-      rv[entry_k] = entry
-    end
-  end
-
-  return(rv)
-end
-
 -- This table is used both to control access to the preferences and to filter preferences results
 local menu_subpages = {
   {id="auth",          label=i18n("prefs.user_authentication"),  advanced=false, pro_only=false, nedge_hidden=false, hidden=(not(prefs.is_users_login_enabled) and not have_nedge), entries={
@@ -165,6 +147,20 @@ local menu_subpages = {
       title       = i18n("prefs.local_host_cache_duration_title"),
       description = i18n("prefs.local_host_cache_duration_description"),
     },
+  }}, {id="dump_settings",  label=i18n("prefs.dump_settings"),   advanced=true,  pro_only=false,  hidden=(prefs.is_dump_flows_enabled == false), entries={
+    toggle_enable_runtime_flows_dump = {
+      title       = i18n("prefs.toggle_enable_runtime_flows_dump_title"),
+      description = i18n("prefs.toggle_enable_runtime_flows_dump_description"),
+    }, toggle_tiny_flows_dump = {
+      title       = i18n("prefs.toggle_tiny_flows_dump_title"),
+      description = i18n("prefs.toggle_tiny_flows_dump_description"),
+    }, max_num_packets_per_tiny_flow = {
+      title       = i18n("prefs.max_num_packets_per_tiny_flow_title"),
+      description = i18n("prefs.max_num_packets_per_tiny_flow_description"),
+    }, max_num_bytes_per_tiny_flow = {
+      title       = i18n("prefs.max_num_bytes_per_tiny_flow_title"),
+      description = i18n("prefs.max_num_bytes_per_tiny_flow_description"),
+    },
   }}, {id="on_disk_ts",    label=i18n("prefs.timeseries"),       advanced=false, pro_only=false,  hidden=false, entries={
     toggle_interface_traffic_rrd_creation = {
       title       = i18n("prefs.toggle_traffic_rrd_creation_title"),
@@ -172,6 +168,9 @@ local menu_subpages = {
     },toggle_local_hosts_ts_creation = {
       title       = i18n("prefs.toggle_local_hosts_ts_creation_title"),
       description = i18n("prefs.toggle_local_hosts_ts_creation_description"),
+    }, toggle_local_hosts_one_way_ts = {
+      title       = i18n("prefs.toggle_local_hosts_one_way_ts_title"),
+      description = i18n("prefs.toggle_local_hosts_one_way_ts_description"),
     }, toggle_l2_devices_traffic_rrd_creation = { -- layer 2 devices
       title       = i18n("prefs.toggle_traffic_rrd_creation_title"),
       description = i18n("prefs.toggle_traffic_rrd_creation_description"),
@@ -223,7 +222,7 @@ local menu_subpages = {
       description = i18n("prefs.influxdb_password_description"),
     }, timeseries_resolution_resolution = {
       title       = i18n("prefs.timeseries_resolution_resolution_title"),
-      description = i18n("prefs.timeseries_resolution_resolution_description"),
+      description = i18n("prefs.timeseries_resolution_resolution_description_2"),
     }, influxdb_query_timeout = {
       title       = i18n("prefs.influxdb_query_timeout_title"),
       description = i18n("prefs.influxdb_query_timeout_description"),
@@ -232,12 +231,6 @@ local menu_subpages = {
     disable_alerts_generation = {
       title       = i18n("prefs.disable_alerts_generation_title"),
       description = i18n("prefs.disable_alerts_generation_description"),
-    }, toggle_remote_to_remote_alerts = {
-      title       = i18n("prefs.toggle_remote_to_remote_alerts_title"),
-      description = i18n("prefs.toggle_remote_to_remote_alerts_description"),
-    }, toggle_ip_reassignment_alerts = {
-       title       = i18n("prefs.toggle_ip_reassignment_title"),
-       description = i18n("prefs.toggle_ip_reassignment_description"),
     }, max_num_alerts_per_entity = {
       title       = i18n("prefs.max_num_alerts_per_entity_title"),
       description = i18n("prefs.max_num_alerts_per_entity_description"),
@@ -248,24 +241,8 @@ local menu_subpages = {
       title       = i18n("prefs.toggle_mysql_check_open_files_limit_title"),
       description = i18n("prefs.toggle_mysql_check_open_files_limit_description"),
       hidden      = (prefs.is_dump_flows_to_mysql_enabled == false),
-    }, toggle_device_first_seen_alert = {
-      title       =  i18n("prefs.toggle_device_first_seen_alert_title"),
-      description = i18n("prefs.toggle_device_first_seen_alert_description"),
-    }, toggle_device_activation_alert = {
-      title       = i18n("prefs.toggle_device_activation_alert_title"),
-      description = i18n("prefs.toggle_device_activation_alert_description"),
-    }, toggle_pool_activation_alert = {
-      title       = i18n("prefs.toggle_pool_activation_alert_title"),
-      description = i18n("prefs.toggle_pool_activation_alert_description"),
-    }, toggle_quota_exceeded_alert = {
-      title       = i18n("prefs.toggle_quota_exceed_alert_title"),
-      description = i18n("prefs.toggle_quota_exceed_alert_description"),
-      hidden      = not ntop.isPro(),
     }
-    
-  }}, {id="ext_alerts",    label=i18n("prefs.alerts_notifications"), advanced=false, hidden=hasAlertsDisabled(), pro_only=false,
-    entries = getAlertEndpointsEntries()
-  }, {id="protocols",     label=i18n("prefs.protocols"),            advanced=false, pro_only=false,  hidden=false, entries={
+  }}, {id="protocols",     label=i18n("prefs.protocols"),            advanced=false, pro_only=false,  hidden=false, entries={
     toggle_top_sites = {
       title       = i18n("prefs.toggle_top_sites_title"),
       description = i18n("prefs.toggle_top_sites_description", {url="https://resources.sei.cmu.edu/asset_files/Presentation/2010_017_001_49763.pdf"})},
@@ -287,23 +264,7 @@ local menu_subpages = {
       title       = i18n("prefs.toggle_host_pools_log_title"),
       description = i18n("prefs.toggle_host_pools_log_description", {product=info["product"]}),
     },
-  }}, {id="flow_db_dump",  label=i18n("prefs.flow_database_dump"),   advanced=true,  pro_only=false,  hidden=(prefs.is_dump_flows_enabled == false), entries={
-    toggle_flow_db_dump_export = {
-      title       = i18n("prefs.toggle_flow_db_dump_export_title"),
-      description = i18n("prefs.toggle_flow_db_dump_export_description"),
-    }, max_num_packets_per_tiny_flow = {
-      title       = i18n("prefs.max_num_packets_per_tiny_flow_title"),
-      description = i18n("prefs.max_num_packets_per_tiny_flow_description"),
-    }, max_num_bytes_per_tiny_flow = {
-      title       = i18n("prefs.max_num_bytes_per_tiny_flow_title"),
-      description = i18n("prefs.max_num_bytes_per_tiny_flow_description"),
-    }, toggle_aggregated_flows_export_limit = {
-      title       = i18n("prefs.toggle_aggregated_flows_export_limit_title"),
-      description = i18n("prefs.toggle_aggregated_flows_export_limit_description"),
-    }, max_num_aggregated_flows_per_export = {
-      title       = i18n("prefs.max_num_aggregated_flows_per_export_title"),
-      description = i18n("prefs.max_num_aggregated_flows_per_export_description"),
-    }
+
   }}, {id="snmp",          label=i18n("prefs.snmp"),                 advanced=true,  pro_only=true,   hidden=false, nedge_hidden=true, entries={
     toggle_snmp_rrds = {
       title       = i18n("prefs.toggle_snmp_rrds_title"),
@@ -314,6 +275,9 @@ local menu_subpages = {
     }, default_snmp_proto_version = {
        title       = i18n("prefs.default_snmp_proto_version_title"),
        description = i18n("prefs.default_snmp_proto_version_description"),
+    }, default_snmp_timeout = {
+      title       = i18n("prefs.default_snmp_timeout_title"),
+      description = i18n("prefs.default_snmp_timeout_description"),
     }, toggle_snmp_debug = {
        title       = i18n("prefs.toggle_snmp_debug_title"),
        description = i18n("prefs.toggle_snmp_debug_description"),
@@ -380,6 +344,9 @@ local menu_subpages = {
     }, toggle_host_mask = {
       title       = i18n("prefs.toggle_host_mask_title"),
       description = i18n("prefs.toggle_host_mask_description"),
+    }, toggle_behaviour_analysis = {
+      title       = i18n("prefs.toggle_behaviour_analysis_title"),
+      description = i18n("prefs.toggle_behaviour_analysis_description"),
     }, topk_heuristic_precision = {
       title       = i18n("prefs.topk_heuristic_precision_title"),
       description = i18n("prefs.topk_heuristic_precision_description"),
