@@ -1,70 +1,53 @@
 <?php
 
 /*
-    Copyright (C) 2014-2016 Deciso B.V.
-    Copyright (C) 2004-2009 Scott Ullrich <sullrich@gmail.com>
-    Copyright (C) 2011 Seth Mos <seth.mos@dds.nl>
-    Copyright (C) 2003-2004 Manuel Kasper <mk@neon1.net>.
-    All rights reserved.
-
-    Redistribution and use in source and binary forms, with or without
-    modification, are permitted provided that the following conditions are met:
-
-    1. Redistributions of source code must retain the above copyright notice,
-       this list of conditions and the following disclaimer.
-
-    2. Redistributions in binary form must reproduce the above copyright
-       notice, this list of conditions and the following disclaimer in the
-       documentation and/or other materials provided with the distribution.
-
-    THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
-    INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
-    AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
-    AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
-    OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-    SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-    INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-    CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-    ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-    POSSIBILITY OF SUCH DAMAGE.
-*/
+ * Copyright (C) 2014-2016 Deciso B.V.
+ * Copyright (C) 2004-2009 Scott Ullrich <sullrich@gmail.com>
+ * Copyright (C) 2011 Seth Mos <seth.mos@dds.nl>
+ * Copyright (C) 2003-2004 Manuel Kasper <mk@neon1.net>
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright notice,
+ *    this list of conditions and the following disclaimer.
+ *
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ *
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
+ * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
+ * AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
+ * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ */
 
 require_once("guiconfig.inc");
 require_once("interfaces.inc");
-require_once("services.inc");
+require_once("plugins.inc.d/dhcpd.inc");
 
-function leasecmp($a, $b) {
-  return strcmp($a[$_GET['order']], $b[$_GET['order']]);
-}
-
-function adjust_gmt($dt) {
-    global $config;
-
-    $dhcpv6leaseinlocaltime == "no";
-    if (is_array($config['dhcpdv6'])) {
-        $dhcpdv6 = $config['dhcpdv6'];
-        foreach ($dhcpdv6 as $dhcpv6leaseinlocaltime) {
-            $dhcpv6leaseinlocaltime = $dhcpv6leaseinlocaltime['dhcpv6leaseinlocaltime'];
-            if ($dhcpv6leaseinlocaltime == "yes") {
-                break;
-            }
+function adjust_utc($dt)
+{
+    foreach (config_read_array('dhcpdv6') as $dhcpdv6) {
+        if (!empty($dhcpdv6['dhcpv6leaseinlocaltime'])) {
+            /* we want local time, so specify this is actually UTC */
+            return strftime('%Y/%m/%d %H:%M:%S', strtotime("{$dt} UTC"));
         }
     }
 
-    $timezone = $config['system']['timezone'];
-    $ts = strtotime($dt . " GMT");
-    if ($dhcpv6leaseinlocaltime == "yes") {
-        $this_tz = new DateTimeZone($timezone);
-        $dhcp_lt = new DateTime(strftime("%I:%M:%S%p", $ts), $this_tz);
-        $offset = $this_tz->getOffset($dhcp_lt);
-        $ts = $ts + $offset;
-        return strftime("%Y/%m/%d %I:%M:%S%p", $ts);
-    } else {
-        return strftime("%Y/%m/%d %H:%M:%S", $ts);
-    }
+    /* lease time is in UTC, here just pretend it's the correct time */
+    return strftime('%Y/%m/%d %H:%M:%S UTC', strtotime($dt));
 }
 
-function remove_duplicate($array, $field) {
+function remove_duplicate($array, $field)
+{
     foreach ($array as $sub) {
         $cmp[] = $sub[$field];
     }
@@ -75,40 +58,53 @@ function remove_duplicate($array, $field) {
     return $new;
 }
 
-function parse_duid($duid_string) {
-    $parsed_duid = array();
-    for ($i=0; $i < strlen($duid_string); $i++) {
+function parse_duid($duid_string)
+{
+    $parsed_duid = [];
+
+    for ($i = 0; $i < strlen($duid_string); $i++) {
         $s = substr($duid_string, $i, 1);
         if ($s == '\\') {
-            $n = substr($duid_string, $i+1, 1);
-            if (($n == '\\') || ($n == '"')) {
-                $parsed_duid[] = sprintf("%02x", ord($n));
+            $n = substr($duid_string, $i + 1, 1);
+            if ($n == '\\' || $n == '"') {
+                $parsed_duid[] = sprintf('%02x', ord($n));
+                $i += 1;
             } elseif (is_numeric($n)) {
-                $parsed_duid[] = sprintf("%02x", octdec(substr($duid_string, $i+1, 3)));
+                $parsed_duid[] = sprintf('%02x', octdec(substr($duid_string, $i + 1, 3)));
                 $i += 3;
             }
         } else {
-            $parsed_duid[] = sprintf("%02x", ord($s));
+            $parsed_duid[] = sprintf('%02x', ord($s));
         }
     }
+
     $iaid = array_slice($parsed_duid, 0, 4);
     $duid = array_slice($parsed_duid, 4);
-    return array($iaid, $duid);
+
+    return [$iaid, $duid];
 }
 
 $interfaces = legacy_config_get_interfaces(array('virtual' => false));
-$leasesfile = services_dhcpdv6_leasesfile();
+$leasesfile = dhcpd_dhcpv6_leasesfile();
 
 if ($_SERVER['REQUEST_METHOD'] === 'GET') {
-    $awk = "/usr/bin/awk";
+    $leases_content = array();
 
-    /* this pattern sticks comments into a single array item */
-    $cleanpattern = "'{ gsub(\"^#.*\", \"\");} { gsub(\"^server-duid.*\", \"\");} { gsub(\";$\", \"\"); print;}'";
-    /* We then split the leases file by } */
-    $splitpattern = "'BEGIN { RS=\"}\";} {for (i=1; i<=NF; i++) printf \"%s \", \$i; printf \"}\\n\";}'";
+    $fin = @fopen($leasesfile, "r");
+    $section = array();
+    if ($fin) {
+        while (($line = fgets($fin, 4096)) !== false) {
+            if (strpos($line, "ia-") !== false) {
+                $section = array();
+            } elseif (strpos($line, "}") === 0 && count($section) > 0) {
+                $output = implode($section, "");
+                $leases_content[] = $output;
+            }
+            $section[] = rtrim($line, "\n;");
+        }
+        fclose($fin);
+    }
 
-    /* stuff the leases file in a proper format into a array by line */
-    exec("/bin/cat {$leasesfile} | {$awk} {$cleanpattern} | {$awk} {$splitpattern} | /usr/bin/grep '^ia-.. '", $leases_content);
     $leases_count = count($leases_content);
     exec("/usr/sbin/ndp -an", $rawdata);
     $ndpdata = array();
@@ -139,11 +135,11 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
         preg_match('/ia-.. "(.*)" { (.*)/ ', $leases_content[$i], $duid_split);
         if (!empty($duid_split[1])) {
             $iaid_duid = parse_duid($duid_split[1]);
-            $entry['iaid'] = hexdec(implode("", array_reverse($iaid_duid[0])));
-            $entry['duid'] = implode(":", $iaid_duid[1]);
-            $data = explode(" ", $duid_split[2]);
+            $entry['iaid'] = hexdec(implode('', array_reverse($iaid_duid[0])));
+            $entry['duid'] = implode(':', $iaid_duid[1]);
+            $data = explode(' ', $duid_split[2]);
         } else {
-            $data = explode(" ", $leases_content[$i]);
+            $data = explode(' ', $leases_content[$i]);
         }
         /* walk the fields */
         $f = 0;
@@ -252,11 +248,11 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
                     $f = $f+2;
                     break;
                 case "client-hostname":
-                    if ($data[$f+1] <> "") {
+                    if ($data[$f+1] != '') {
                         $entry['hostname'] = preg_replace('/"/','',$data[$f+1]);
                     } else {
                         $hostname = gethostbyaddr($entry['ip']);
-                        if ($hostname <> "") {
+                        if ($hostname != '') {
                             $entry['hostname'] = $hostname;
                         }
                     }
@@ -301,8 +297,8 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
                 $slease['duid'] = $static['duid'];
                 $slease['start'] = "";
                 $slease['end'] = "";
-                $slease['hostname'] = htmlentities($static['hostname']);
-                $slease['descr'] = htmlentities($static['descr']);
+                $slease['hostname'] = $static['hostname'];
+                $slease['descr'] = $static['descr'];
                 $slease['act'] = "static";
                 if (in_array($slease['ip'], array_keys($ndpdata))) {
                     $slease['online'] = 'online';
@@ -316,13 +312,13 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
     }
 
     if ($_GET['order']) {
-        usort($leases, "leasecmp");
+        usort($leases, function ($a, $b) {
+            return strcmp($a[$_GET['order']], $b[$_GET['order']]);
+        });
     }
 } elseif ($_SERVER['REQUEST_METHOD'] === 'POST') {
     if (!empty($_POST['deleteip']) && is_ipaddr($_POST['deleteip'])) {
-        // delete dhcp lease
-        /* Stop DHCPD */
-        killbyname("dhcpd");
+        killbypid('/var/dhcpd/var/run/dhcpdv6.pid', 'TERM', true);
         $fin = @fopen($leasesfile, "r");
         $fout = @fopen($leasesfile.".new", "w");
         if ($fin) {
@@ -361,8 +357,8 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
             fclose($fout);
             @unlink($leasesfile);
             @rename($leasesfile.".new", $leasesfile);
-            /* Restart DHCP Service */
-            services_dhcpd_configure();
+
+            dhcpd_dhcp_configure(false, 'inet6');
         }
     }
     exit;
@@ -372,6 +368,17 @@ $service_hook = 'dhcpd6';
 
 include("head.inc");
 
+$leases_count = 0;
+
+foreach ($leases as $data) {
+   if (!($data['act'] == 'active' || $data['act'] == 'static' || $_GET['all'] == 1)) {
+       continue;
+   }
+   $leases_count++;
+}
+
+$gentitle_suffix = " ($leases_count)";
+legacy_html_escape_form_data($leases);
 ?>
 <body>
   <script>
@@ -408,9 +415,9 @@ if (count($pools) > 0):?>
               <tr>
                   <td><?=$data['name'];?></td>
                   <td><?=$data['mystate'];?></td>
-                  <td><?=adjust_gmt($data['mydate']);?></td>
+                  <td><?=adjust_utc($data['mydate']);?></td>
                   <td><?=$data['peerstate'];?></td>
-                  <td><?=adjust_gmt($data['peerdate']);?></td>
+                  <td><?=adjust_utc($data['peerdate']);?></td>
               </tr>
 
 <?php
@@ -427,7 +434,7 @@ endif;?>
             <table class="table table-striped">
               <thead>
                 <tr>
-                    <th><?=gettext("Interface"); ?></td>
+                    <th><?=gettext("Interface"); ?></th>
                     <th><?=gettext("IPv6 address"); ?></th>
                     <th><?=gettext("IAID"); ?></th>
                     <th><?=gettext("DUID"); ?></th>
@@ -444,6 +451,9 @@ endif;?>
 <?php
               $mac_man = json_decode(configd_run("interface list macdb json"), true);
               foreach ($leases as $data):
+                if (!($data['act'] == 'active' || $data['act'] == 'static' || $_GET['all'] == 1)) {
+                    continue;
+                }
                 if ($data['act'] == "static") {
                     foreach ($config['dhcpdv6'] as $dhcpif => $dhcpifconf) {
                         if (isset($dhcpifconf['staticmap'])) {
@@ -475,25 +485,29 @@ endif;?>
                     <?=!empty($ndpdata[$data['ip']]) ? $ndpdata[$data['ip']]['mac'] : "";?>
                   </td>
                   <td><?=htmlentities($data['descr']);?></td>
-                  <td><?=$data['type'] != "static" ? adjust_gmt($data['start']) : "";?></td>
-                  <td><?=$data['type'] != "static" ? adjust_gmt($data['end']) : "";?></td>
-                  <td><?=$data['online'];?></td>
+                  <td><?=$data['type'] != "static" ? adjust_utc($data['start']) : "";?></td>
+                  <td><?=$data['type'] != "static" ? adjust_utc($data['end']) : "";?></td>
+                  <td>
+                    <i class="fa fa-<?=$data['online']=='online' ? 'signal' : 'ban';?>" title="<?=$data['online'];?>" data-toggle="tooltip"></i>
+                  </td>
                   <td><?=$data['act'];?></td>
                   <td class="text-nowrap">
-<?php
-                      if ($data['type'] == "dynamic"):?>
+<?php if (!empty($config['interfaces'][$data['if']])): ?>
+<?php if (empty($config['interfaces'][$data['if']]['virtual']) && isset($config['interfaces'][$data['if']]['enable'])): ?>
+<?php if (is_ipaddrv6($config['interfaces'][$data['if']]['ipaddrv6']) || !empty($config['interfaces'][$data['if']]['dhcpd6track6allowoverride'])): ?>
+<?php if ($data['type'] == 'dynamic'): ?>
                         <a class="btn btn-default btn-xs" href="services_dhcpv6_edit.php?if=<?=$data['if'];?>&amp;duid=<?=$data['duid'];?>&amp;hostname=<?=$data['hostname'];?>">
-                          <i class="fa fa-plus fa-fw" alt="add"></i>
+                          <i class="fa fa-plus fa-fw"></i>
                         </a>
-<?php
-                      endif;?>
-<?php
-                    if (($data['type'] == "dynamic") && ($data['online'] != "online")):?>
+<?php if ($data['online'] != 'online'): ?>
                     <a class="act_delete btn btn-default btn-xs" href="#" data-deleteip="<?=$data['ip'];?>" title="<?= html_safe(gettext('Delete')) ?>" data-toggle="tooltip">
                       <i class="fa fa-trash fa-fw"></i>
                     </a>
-<?php
-                    endif;?>
+<?php endif ?>
+<?php endif ?>
+<?php endif ?>
+<?php endif ?>
+<?php endif ?>
                   </td>
                 </tr>
 <?php
@@ -531,8 +545,8 @@ endif;?>
                   </td>
                   <td><?=$data['iaid'];?></td>
                   <td><?=$data['duid'];?></td>
-                  <td><?=$data['type'] != "static" ? adjust_gmt($data['start']) : "";?></td>
-                  <td><?=$data['type'] != "static" ? adjust_gmt($data['end']) : "";?></td>
+                  <td><?=$data['type'] != "static" ? adjust_utc($data['start']) : "";?></td>
+                  <td><?=$data['type'] != "static" ? adjust_utc($data['end']) : "";?></td>
                   <td><?=$data['act'];?></td>
                 </tr>
 <?php
@@ -548,18 +562,19 @@ endif;?>
           <input type="hidden" name="order" value="<?=htmlspecialchars($_GET['order']);?>" />
           <?php if ($_GET['all']): ?>
           <input type="hidden" name="all" value="0" />
-          <input type="submit" class="btn btn-default" value="<?=gettext("Show active and static leases only"); ?>" />
+          <input type="submit" class="btn btn-default" value="<?= html_safe(gettext('Show active and static leases only')) ?>" />
           <?php else: ?>
           <input type="hidden" name="all" value="1" />
-          <input type="submit" class="btn btn-default" value="<?=gettext("Show all configured leases"); ?>" />
+          <input type="submit" class="btn btn-default" value="<?= html_safe(gettext('Show all configured leases')) ?>" />
           <?php endif; ?>
           </form>
           <?php if ($leases == 0): ?>
-          <p><strong><?=gettext("No leases file found. Is the DHCP server active"); ?>?</strong></p>
+          <p><?= gettext('No leases file found. Is the DHCP server active?') ?></p>
           <?php endif; ?>
       </section>
     </div>
   </div>
 </section>
+<?php
 
-<?php include("foot.inc"); ?>
+include("foot.inc");
