@@ -56,6 +56,7 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
     $pconfig['disableintegratedauth'] = !empty($config['system']['disableintegratedauth']);
     $pconfig['sudo_allow_wheel'] = $config['system']['sudo_allow_wheel'];
     $pconfig['sudo_allow_group'] = isset($config['system']['sudo_allow_group']) ? $config['system']['sudo_allow_group'] : null;
+    $pconfig['user_allow_gen_token'] = isset($config['system']['user_allow_gen_token']) ? explode(",", $config['system']['user_allow_gen_token']) : [];
     $pconfig['nodnsrebindcheck'] = isset($config['system']['webgui']['nodnsrebindcheck']);
     $pconfig['nohttpreferercheck'] = isset($config['system']['webgui']['nohttpreferercheck']);
     $pconfig['althostnames'] = $config['system']['webgui']['althostnames'];
@@ -239,13 +240,11 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
         $config['system']['ssh']['noauto'] = 1;
 
         $config['system']['ssh']['interfaces'] = !empty($pconfig['sshinterfaces']) ? implode(',', $pconfig['sshinterfaces']) : null;
-
         $config['system']['ssh']['kex'] = !empty($pconfig['ssh-kex']) ? implode(',', $pconfig['ssh-kex']) : null;
         $config['system']['ssh']['ciphers'] = !empty($pconfig['ssh-ciphers']) ? implode(',', $pconfig['ssh-ciphers']) : null;
         $config['system']['ssh']['macs'] = !empty($pconfig['ssh-macs']) ? implode(',', $pconfig['ssh-macs']) : null;
         $config['system']['ssh']['keys'] = !empty($pconfig['ssh-keys']) ? implode(',', $pconfig['ssh-keys']) : null;
 
-
         if (!empty($pconfig['enablesshd'])) {
             $config['system']['ssh']['enabled'] = 'enabled';
         } elseif (isset($config['system']['ssh']['enabled'])) {
@@ -264,6 +263,12 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
             unset($config['system']['sudo_allow_group']);
         }
 
+        if (!empty($pconfig['user_allow_gen_token'])) {
+            $config['system']['user_allow_gen_token'] = implode(",", $pconfig['user_allow_gen_token']);
+        } elseif (isset($config['system']['user_allow_gen_token'])) {
+            unset($config['system']['user_allow_gen_token']);
+        }
+
         if (!empty($pconfig['sshpasswordauth'])) {
             $config['system']['ssh']['passwordauth'] = true;
         } elseif (isset($config['system']['ssh']['passwordauth'])) {
@@ -373,7 +378,7 @@ $(document).ready(function() {
                  btnCancelLabel: '<?= html_safe(gettext('Use the default')) ?>',
                  callback: function(result) {
                      if (!result) {
-                         $('#webguiinterface option:selected').removeAttr('selected');
+                         $('#webguiinterface option:selected').prop('selected', false);
                          $('#webguiinterface').selectpicker('refresh');
                          $.webguiinterface_warned = 0;
                      }
@@ -381,7 +386,7 @@ $(document).ready(function() {
              });
          }
      });
-     $.webguiinterface_warned = 0;
+     $.webguiinterface_warned = $('#webguiinterface option:selected').length ? 1 : 0;
 
  <?php
     if (isset($restart_webgui) && $restart_webgui): ?>
@@ -928,6 +933,21 @@ $(document).ready(function() {
                   </div>
                 </td>
               </tr>
+              <tr>
+                <td><a id="help_for_user_allow_gen_token" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a> <?= gettext('User OTP seed') ?></td>
+                <td>
+                  <select name="user_allow_gen_token[]" multiple=multiple class="selectpicker">
+<?php foreach ($a_group as $group): ?>
+                      <option value="<?= html_safe($group['name']) ?>" <?= in_array($group['name'], $pconfig['user_allow_gen_token'] ?? []) ? 'selected="selected"' : '' ?>>
+                        <?= html_safe($group['name']) ?>
+                      </option>
+<?php endforeach ?>
+                  </select>
+                  <div class="hidden" data-for="help_for_user_allow_gen_token">
+                    <?= gettext('Permit users to generate their own OTP seed in the password page.') ?>
+                  </div>
+                </td>
+              </tr>
             </table>
           </div>
           <div class="content-box tab-content table-responsive">
